@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2022-02-19
+ *  Updated   : 2023-07-26
  *  Notes     :
  *
- * Copyright (c) 1997-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2023 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -582,12 +582,12 @@ extern constext txt_command_failed[];
 extern constext txt_cook_article_failed_exiting[];
 extern constext txt_confirm_select_on_exit[];
 #ifdef NNTP_ABLE
-	extern constext txt_connecting[];
 	extern constext txt_connecting_port[];
 #endif /* NNTP_ABLE */
 #if defined(NNTP_ABLE) && !defined(INET6)
 	extern constext txt_connection_to[];
 #endif /* NNTP_ABLE && !INET6 */
+extern constext txt_connection_info[];
 extern constext txt_copyright_notice[];
 extern constext txt_cr[];
 extern constext txt_creating_active[];
@@ -615,7 +615,9 @@ extern constext txt_enter_next_unread_group[];
 extern constext txt_enter_option_num[];
 extern constext txt_enter_range[];
 extern constext txt_error_approved[];
-extern constext txt_error_asfail[];
+#ifndef NDEBUG
+	extern constext txt_error_asfail[];
+#endif /* ! NDEBUG */
 extern constext txt_error_bad_approved[];
 extern constext txt_error_bad_from[];
 extern constext txt_error_bad_msgidfqdn[];
@@ -626,6 +628,9 @@ extern constext txt_error_bad_to[];
 	extern constext txt_error_couldnt_dotlock[];
 	extern constext txt_error_couldnt_lock[];
 #endif /* NO_LOCKING */
+#if defined(NNTP_ABLE) && defined(USE_ZLIB)
+	extern constext txt_error_compression_auth[];
+#endif /* NNTP_ABLE && USE_ZLIB */
 extern constext txt_error_copy_fp[];
 extern constext txt_error_corrupted_file[];
 extern constext txt_error_fseek[];
@@ -939,6 +944,7 @@ extern constext txt_help_select_search_group_forwards[];
 extern constext txt_help_select_subscribe[];
 extern constext txt_help_select_subscribe_pattern[];
 extern constext txt_help_select_sync_with_active[];
+extern constext txt_help_global_connection_info[];
 extern constext txt_help_select_toggle_descriptions[];
 extern constext txt_help_select_toggle_read_groups[];
 extern constext txt_help_select_unsubscribe[];
@@ -978,6 +984,7 @@ extern constext txt_info_add_kill[];
 extern constext txt_info_add_select[];
 extern constext txt_info_all_parts_tagged[];
 extern constext txt_info_all_parts_untagged[];
+extern constext txt_info_building_ref_tree[];
 extern constext txt_info_do_postpone[];
 extern constext txt_info_enter_valid_character[];
 extern constext txt_info_missing_part[];
@@ -1011,6 +1018,9 @@ extern constext txt_kill_time[];
 extern constext txt_last[];
 extern constext txt_last_resp[];
 extern constext txt_lines[];
+#if defined(NNTP_ABLE) && defined(DEBUG)
+	extern constext txt_log_data_hidden[];
+#endif /* NNTP_ABLE && DEBUG */
 extern constext txt_lookup_func_not_available[];
 extern constext txt_lookup_func_not_nntp[];
 #ifdef NNTP_ABLE
@@ -1119,7 +1129,9 @@ extern constext txt_options_menu_com[];
 extern constext txt_out_of_memory[];
 extern constext txt_pcre_error_at[];
 extern constext txt_pcre_error_num[];
-extern constext txt_pcre_error_text[];
+#ifndef HAVE_LIB_PCRE2
+	extern constext txt_pcre_error_text[];
+#endif /* !HAVE_LIB_PCRE2 */
 extern constext txt_post_a_followup[];
 extern constext txt_post_error_ask_postpone[];
 extern constext txt_post_history_menu[];
@@ -1179,9 +1191,9 @@ extern constext txt_reading_newsrc[];
 	extern constext txt_reconnect_to_news_server[];
 #endif /* NNTP_ABLE */
 extern constext txt_refs_line_only[];
-#ifdef HAVE_GETTIMEOFDAY
+#if defined(HAVE_CLOCK_GETTIME) || defined(HAVE_GETTIMEOFDAY)
 	extern constext txt_remaining[];
-#endif /* HAVE_GETTIMEOFDAY */
+#endif /* HAVE_CLOCK_GETTIME || HAVE_GETTIMEOFDAY */
 extern constext txt_remove_bogus[];
 extern constext txt_removed_rule[];
 extern constext txt_rename_error[];
@@ -1265,9 +1277,6 @@ extern constext txt_thread_upper[];
 extern constext txt_thread_com[];
 extern constext txt_thread_marked_as_deselected[];
 extern constext txt_thread_marked_as_selected[];
-#if 0 /* unused */
-	extern constext txt_thread_plural[];
-#endif /* 0 */
 extern constext txt_thread_singular[];
 extern constext txt_thread_x_of_n[];
 extern constext txt_threading_arts[];
@@ -1305,6 +1314,9 @@ extern constext txt_url_select[];
 extern constext txt_url_done[];
 extern constext txt_usage_catchup[];
 extern constext txt_usage_check_for_unread_news[];
+#ifdef USE_ZLIB
+	extern constext txt_usage_compress[];
+#endif /* USE_ZLIB */
 #ifdef DEBUG
 	extern constext txt_usage_debug[];
 #endif /* DEBUG */
@@ -1313,13 +1325,17 @@ extern constext txt_usage_dont_save_files_on_quit[];
 extern constext txt_usage_dont_show_descriptions[];
 #ifdef NNTP_ABLE
 	extern constext txt_usage_force_authentication[];
-#ifdef INET6
+#	ifdef INET6
 	extern constext txt_usage_force_ipv4[];
 	extern constext txt_usage_force_ipv6[];
-#endif /* INET6 */
+#	endif /* INET6 */
 	extern constext txt_usage_newsserver[];
 	extern constext txt_usage_port[];
 	extern constext txt_usage_read_news_remotely[];
+#	ifdef NNTPS_ABLE
+	extern constext txt_usage_use_insecure_nntps[];
+	extern constext txt_usage_use_nntps[];
+#	endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
 extern constext txt_usage_getart_limit[];
 extern constext txt_usage_help_information[];
@@ -1352,6 +1368,10 @@ extern constext txt_useful_with_batch_mode[];
 extern constext txt_useful_with_batch_or_debug_mode[];
 extern constext txt_useless_combination[];
 extern constext txt_use_mime[];
+#ifdef NNTPS_ABLE
+	extern constext txt_valid_not_after[];
+	extern constext txt_valid_not_before[];
+#endif /* NNTPS_ABLE */
 extern constext txt_value_out_of_range[];
 extern constext txt_view_attachment[];
 extern constext txt_warn_art_line_too_long[];
@@ -1376,6 +1396,7 @@ extern constext txt_warn_references_but_no_re[];
 extern constext txt_warn_sig_too_long[];
 extern constext txt_warn_suspicious_mail[];
 extern constext txt_warn_update[];
+extern constext txt_warn_unprintable_char[];
 extern constext txt_warn_unrecognized_version[];
 extern constext txt_warn_wrong_sig_format[];
 extern constext txt_warn_xref_not_supported[];
@@ -1519,6 +1540,9 @@ extern t_bool dangerous_signal_exit; /* TRUE if SIGHUP, SIGTERM, SIGUSR1 */
 #ifdef NNTP_ABLE
 	extern t_bool did_reconnect;
 	extern t_bool reconnected_in_last_get_server;
+#	ifdef USE_ZLIB
+		extern t_bool use_compress;
+#	endif /* USE_ZLIB */
 #endif /* NNTP_ABLE */
 extern t_bool disable_gnksa_domain_check;
 extern t_bool disable_sender;
@@ -1541,6 +1565,8 @@ extern t_bool read_saved_news;
 extern t_bool reread_active_for_posted_arts;
 extern t_bool show_description;
 extern t_bool show_subject;
+extern t_bool use_nntps;
+extern t_bool insecure_nntps;
 extern t_bool batch_mode;
 extern int verbose;
 extern t_bool xref_supported;
@@ -1703,6 +1729,10 @@ extern constext txt_processing_saved_arts[];
 #ifdef NNTP_ABLE
 	extern constext txt_nntp_ok_goodbye[];
 	extern unsigned short nntp_tcp_port;
+	extern unsigned short nntp_tcp_default_port;
+#ifdef NNTPS_ABLE
+	extern unsigned short nntps_tcp_default_port;
+#endif /* NNTPS_ABLE */
 	extern t_bool force_auth_on_conn_open;
 #endif /* NNTP_ABLE */
 
@@ -1850,6 +1880,9 @@ extern struct opttxt txt_strip_newsrc;
 extern struct opttxt txt_strip_re_regex;
 extern struct opttxt txt_strip_was_regex;
 extern struct opttxt txt_strokes_regex;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern struct opttxt txt_suppress_soft_hyphens;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 extern struct opttxt txt_tex2iso_conv;
 extern struct opttxt txt_thread_articles;
 extern struct opttxt txt_thread_perc;
@@ -1857,6 +1890,9 @@ extern struct opttxt txt_thread_catchup_on_exit;
 extern struct opttxt txt_thread_format;
 extern struct opttxt txt_thread_score;
 extern struct opttxt txt_trim_article_body;
+#ifdef NNTPS_ABLE
+extern struct opttxt txt_tls_ca_cert_file;
+#endif /* NNTPS_ABLE */
 extern struct opttxt txt_underscores_regex;
 extern struct opttxt txt_unlink_article;
 extern struct opttxt txt_url_handler;
@@ -1914,9 +1950,9 @@ extern struct opttxt txt_xpost_quote_format;
 	extern struct opttxt txt_col_markslash;
 	extern struct opttxt txt_col_markstroke;
 #endif /* HAVE_COLOR */
-#ifdef HAVE_ICONV_OPEN_TRANSLIT
+#if defined(HAVE_ICONV_OPEN_TRANSLIT) && defined(CHARSET_CONVERSION)
 	extern struct opttxt txt_translit;
-#endif /* HAVE_ICONV_OPEN_TRANSLIT */
+#endif /* HAVE_ICONV_OPEN_TRANSLIT && CHARSET_CONVERSION */
 #ifdef HAVE_KEYPAD
 	extern struct opttxt txt_use_keypad;
 #endif /* HAVE_KEYPAD */
