@@ -3,10 +3,10 @@
  *  Module    : nntplib.h
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2018-02-10
+ *  Updated   : 2022-12-23
  *  Notes     : nntp.h 1.5.11/1.6 with extensions for tin
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,9 +47,12 @@
 
 #define NNTP_TCP_NAME	"nntp"
 #define NNTP_TCP_PORT	"119"
+#define NNTPS_TCP_NAME	"nntps"
+#define NNTPS_TCP_PORT	"563"
 
 /*
  * # times to try and reconnect to server after timeout
+ * TODO: make this configurable via tinrc
  */
 #define NNTP_TRY_RECONNECT		2
 
@@ -121,7 +124,7 @@
 #define NEED_AUTHDATA_SASL	383 /* continue with SASL exchange */
 
 #define ERR_GOODBYE		400	/* Have to hang up for some reason */
-#define ERR_STATE		401 /* The client must change the state of the connection */
+#define ERR_CONNSTATE		401 /* The client must change the state of the connection */
 #define ERR_COMPRESS	403	/* Unable to activate compression */
 #define ERR_NOGROUP		411	/* No such newsgroup */
 #define ERR_NCING		412	/* Not currently in newsgroup */
@@ -217,7 +220,11 @@ struct t_capabilities {
 	enum sasl_types sasl;			/* SASL_NONE, SASL_PLAIN, SASL_CRAM_MD5, SASL_DIGEST_MD5, SASL_GSSAPI, SASL_EXTERNAL, SASL_OTP, SASL_NTLM, SASL_LOGIN */
 	t_bool compress:1;				/* COMPRESS */
 	enum c_algorithms compress_algorithm;	/* COMPRESS_NONE, COMPRESS_DEFLATE */
+#if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+	t_artnum maxartnum;				/* MAXARTNUM indicating article numbers >=2^31 */
+#endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
 #if 0
+	/* we don't do MODE STREAM, CHECK, TAKETHIS or IHAVE*/
 	t_bool streaming:1;				/* STREAMING: "MODE STREAM", "CHECK", "TAKETHIS" */
 	t_bool ihave:1;					/* IHAVE: "IHAVE" */
 #endif /* 0 */
