@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2022-01-29
+ *  Updated   : 2023-08-23
  *  Notes     :
  *
- * Copyright (c) 1997-2022 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2023 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -270,9 +270,6 @@ extern void postinit_regexp(void);
 #ifdef HAVE_COLOR
 	extern void postinit_colors(int last_color);
 #endif /* HAVE_COLOR */
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	extern t_bool utf8_pcre(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* joinpath.c */
 extern void joinpath(char *result, size_t result_size, const char *dir, const char *file);
@@ -341,6 +338,9 @@ extern void *my_realloc1(const char *file, int line, void *p, size_t size);
 #ifndef USE_CURSES
 	extern void init_screen_array(t_bool allocate);
 #endif /* !USE_CURSES */
+#if !defined(HAVE_MEMMOVE) && !defined(HAVE_BCOPY)
+	extern void my_memmove(void *dest, const void *src, size_t n);
+#endif /* !HAVE_MEMMOVE && !HAVE_BCOPY */
 
 /* mimetypes.c */
 extern void lookup_mimetype(const char *ext, t_part *part);
@@ -357,6 +357,7 @@ extern char *strip_line(char *line);
 extern const char *eat_re(char *s, t_bool eat_was);
 extern const char *get_val(const char *env, const char *def);
 extern const char *gnksa_strerror(int errcode);
+extern const char *validate_charset(const char *charset);
 extern int gnksa_check_from(char *from);
 extern int gnksa_split_from(const char *from, char *address, char *realname, int *addrtype);
 extern int get_initials(struct t_article *art, char *s, int maxsize);
@@ -376,7 +377,9 @@ extern t_bool invoke_cmd(const char *nam);
 extern t_bool invoke_editor(const char *filename, int lineno, struct t_group *group);
 extern t_bool mail_check(const char *mailbox_name);
 extern void append_file(char *old_filename, char *new_filename);
-extern _Noreturn void asfail(const char *file, int line, const char *cond);
+#ifndef NDEBUG
+	extern _Noreturn void asfail(const char *file, int line, const char *cond);
+#endif /* ! NDEBUG */
 extern void base_name(const char *fullpath, char *file);
 extern void cleanup_tmp_files(void);
 extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, char *initl, t_bool raw_data);
@@ -385,11 +388,15 @@ extern void dir_name(const char *fullpath, char *dir);
 extern void draw_mark_selected(int i);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
+extern void show_connection_page(const int level, const char *title);
 extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path, size_t group_path_len);
 extern void make_group_path(const char *name, char *path);
 extern void process_charsets(char **line, size_t *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
 extern void read_input_history_file(void);
 extern void rename_file(const char *old_filename, const char *new_filename);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern void remove_soft_hyphens(char *line);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 extern void show_inverse_video_status(void);
 extern void strip_name(const char *from, char *address);
 extern _Noreturn void tin_done(int ret, const char *fmt, ...);
@@ -435,8 +442,6 @@ extern void thd_mark_unread(struct t_group *group, long thread);
 extern void set_default_bitmap(struct t_group *group);
 
 /* nntplib.c */
-extern FILE *get_nntp_fp(FILE *fp);
-/* extern FILE *get_nntp_wr_fp(FILE *fp); */
 extern char *getserverbyfile(const char *file);
 extern int nntp_open(void);
 extern void nntp_close(t_bool send_no_quit);
@@ -449,8 +454,26 @@ extern void nntp_close(t_bool send_no_quit);
 	extern int new_nntp_command(const char *command, int success, char *message, size_t mlen);
 	extern void put_server(const char *string);
 	extern void u_put_server(const char *string);
+	extern int fgetc_server(FILE *stream);
+	extern int ungetc_server(int c, FILE *stream);
+	extern int nntp_conninfo(FILE *stream);
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+	extern void set_maxartnum(t_bool reconnect);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
 #endif /* NNTP_ABLE */
 
+/* nntps.c */
+	extern int tintls_init(void);
+	extern void tintls_exit(void);
+#if defined(NNTP_ABLE) && defined(NNTPS_ABLE)
+	extern int tintls_open(const char *servername, int fd, void **session_ctx);
+	extern int tintls_close(void *session_ctx);
+	extern int tintls_handshake(void *session_ctx);
+	extern ssize_t tintls_read(void *session_ctx, void *buf, size_t count);
+	extern ssize_t tintls_write(void *session_ctx, const void *buf, size_t count);
+	extern int tintls_conninfo(void *session_ctx, FILE *stream);
+#endif /* NNTP_ABLE && NNTPS_ABLE */
+
 /* nrctbl.c */
 extern t_bool get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const char *nntpserver_name);
 #ifdef NNTP_ABLE
@@ -469,10 +492,10 @@ extern void show_menu_help(const char *help_message);
 /* page.c */
 extern int show_page(struct t_group *group, int start_respnum, int *threadnum);
 extern void display_info_page(int part);
-extern void draw_page(const char *group, int part);
+extern void draw_page(int part);
 extern void info_pager(FILE *info_fh, const char *title, t_bool wrap_at_ends);
 extern void resize_article(t_bool wrap_lines, t_openartinfo *artinfo);
-extern void toggle_raw(struct t_group *group);
+extern void toggle_raw(void);
 
 /* parsdate.y */
 extern time_t parsedate(char *p, TIMEINFO *now);
@@ -555,9 +578,17 @@ extern void free_msgids(void);
 extern void thread_by_reference(void);
 
 /* regex.c */
-extern t_bool compile_regex(const char *regex, struct regex_cache *cache, int options);
+extern t_bool compile_regex(const char *regex, struct regex_cache *cache, REGEX_OPTIONS options);
 extern t_bool match_regex(const char *string, char *pattern, struct regex_cache *cache, t_bool icase);
+extern int match_regex_ex(const char *string, int length, int offset, REGEX_OPTIONS options, struct regex_cache *regex);
+
+extern REGEX_NOFFSET regex_get_ovector_count(struct regex_cache *regex);
+extern REGEX_SIZE *regex_get_ovector_pointer(struct regex_cache *regex);
+
 extern void highlight_regexes(int row, struct regex_cache *regex, int color);
+extern void regex_cache_init(struct regex_cache *regex);
+extern void regex_cache_destroy(struct regex_cache *regex);
+extern t_bool regex_use_utf8(void);
 
 /* rfc1524.c */
 extern t_mailcap *get_mailcap_entry(t_part *part, const char *path);
@@ -592,8 +623,8 @@ extern void compose_mail_mime_forwarded(const char *filename, FILE *articlefp, t
 extern void compose_mail_text_plain(const char *filename, struct t_group *group);
 
 /* save.c */
-extern int check_start_save_any_news(int function, t_bool catchup);
-extern t_bool create_path(const char *path);
+extern int check_start_save_any_news(int function, t_bool catchup, int num_cmd_line_groups);
+extern int create_path(const char *path);
 extern t_bool post_process_files(t_function proc_type_func, t_bool auto_delete);
 extern t_bool save_and_process_art(t_openartinfo *artinfo, t_bool is_mailbox, const char *inpath, int max, t_bool post_process);
 extern t_part *get_part(int n);
@@ -617,6 +648,8 @@ extern void show_title(const char *title);
 extern void spin_cursor(void);
 extern void stow_cursor(void);
 extern void wait_message(unsigned int sdelay, const char *fmt, ...);
+extern void open_msglog(void);
+extern void close_msglog(void);
 
 /* search.c */
 extern enum option_enum search_config(t_bool forward, t_bool repeat, enum option_enum current, enum option_enum last);
@@ -769,7 +802,7 @@ extern void upgrade_prompt_quit(struct t_version *upgrade, const char *file);
 
 /* wildmat.c */
 extern t_bool wildmat(const char *text, char *p, t_bool icase);
-extern t_bool wildmatpos(const char *text, char *p, t_bool icase, int *srch_offsets, int srch_offsets_size);
+extern t_bool wildmatpos(const char *text, char *p, t_bool icase, REGEX_SIZE *srch_offsets, REGEX_NOFFSET srch_offsets_size);
 
 /* xface.c */
 #ifdef XFACE_ABLE
