@@ -3,7 +3,7 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2022-09-20
+ *  Updated   : 2023-05-10
  *  Notes     : #include files, #defines & struct's
  *
  * Copyright (c) 1997-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -594,11 +594,11 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #ifdef HAVE_LONG_FILE_NAMES
 #	define PATH_PART	"part"
 #	define PATH_PATCH	"patch"
-#	define INDEX_LOCK	"tin.%s.LCK"
+#	define INDEX_LOCK	"tin.%.256s.LCK"
 #else
 #	define PATH_PART	""
 #	define PATH_PATCH	"p"
-#	define INDEX_LOCK	"%s.LCK"
+#	define INDEX_LOCK	"%.10s.LCK"
 #endif /* HAVE_LONG_FILE_NAMES */
 
 /* inews.c:submit_inews() and save.c:save_and_process_art() */
@@ -2225,9 +2225,6 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 #	define MAX_SORT_FUNCS 1
 #endif /* !USE_HEAPSORT */
 
-/* Separator between dir part of path & the filename */
-#define DIRSEP	'/'
-
 /*
  * mouse buttons for use in xterm
  */
@@ -2396,7 +2393,7 @@ typedef FILE TCP;
 #endif /* !NNTPLIB_H */
 
 /*
- * rfc1521/rfc1522 interface
+ * RFC 1521 / RFC 1522 interface
  */
 typedef void (*BodyPtr) (char *, FILE *, int);
 
@@ -2541,6 +2538,15 @@ extern struct tm *localtime(time_t *);
 #	define vsnprintf	plp_vsnprintf
 #endif /* HAVE_VSNPRINTF */
 
+#ifdef HAVE_MEMMOVE
+#	define my_memmove memmove
+#else
+#	ifdef HAVE_BCOPY
+#		define my_memmove(d,s,n) bcopy(s,d,n)
+	/* else memory.c:my_memmove() */
+#	endif /* HAVE_BCOPY */
+#endif /* HAVE_MEMMOVE */
+
 /* gcc-specific attributes */
 #if defined(__GNUC__) && !defined(__cplusplus) && !defined(__APPLE_CC__) && !defined(__NeXT__)
 #	if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 6)
