@@ -3,10 +3,10 @@
  *  Module    : tinrc.h
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-04-13
- *  Updated   : 2021-07-02
+ *  Updated   : 2023-10-29
  *  Notes     :
  *
- * Copyright (c) 1999-2022 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1999-2023 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -113,9 +113,9 @@ struct t_config {
 		int mm_network_charset;				/* MIME charset */
 #	endif /* !CHARSET_CONVERSION */
 	char mm_local_charset[LEN];		/* display charset, not a rc/Menu-option anymore -> should be moved elsewhere */
-#	ifdef HAVE_ICONV_OPEN_TRANSLIT
+#	if defined(HAVE_ICONV_OPEN_TRANSLIT) && defined(CHARSET_CONVERSION)
 		t_bool translit;						/* use //TRANSLIT */
-#	endif /* HAVE_ICONV_OPEN_TRANSLIT */
+#	endif /* HAVE_ICONV_OPEN_TRANSLIT && CHARSET_CONVERSION */
 	char news_headers_to_display[LEN];	/* which headers to display */
 	char news_headers_to_not_display[LEN];	/* which headers to not display */
 	char news_quote_format[LEN];
@@ -137,6 +137,9 @@ struct t_config {
 	char verbatim_end_regex[LEN];			/* regex used to find the end of a verbatim block */
 	char savedir[PATH_LEN];				/* directory to save articles to */
 	char spamtrap_warning_addresses[LEN];
+#	ifdef NNTPS_ABLE
+		char tls_ca_cert_file[PATH_LEN];	/* file containing trusted CA certificates */
+#	endif /* NNTPS_ABLE */
 	char url_handler[LEN];				/* Helper app for opening URL's */
 	char xpost_quote_format[LEN];
 	int filter_days;					/* num of days an article filter can be active */
@@ -170,6 +173,7 @@ struct t_config {
 	int score_kill;						/* default score for "kill" filter rules */
 	int score_select;					/* default score for "hot" filter rules */
 	int trim_article_body;				/* remove unnecessary blank lines */
+	int show_help_mail_sign;			/* show help/mail sign in level title */
 #	ifdef HAVE_COLOR
 		int col_back;						/* standard background color */
 		int col_from;						/* color of sender (From:) */
@@ -249,6 +253,9 @@ struct t_config {
 		t_bool strip_blanks;
 #	endif /* !USE_CURSES */
 	t_bool strip_newsrc;
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		t_bool suppress_soft_hyphens;	/* set TRUE to remove soft hyphens (U+00AD) from articles */
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	t_bool tex2iso_conv;			/* convert "a to Umlaut-a */
 	t_bool thread_catchup_on_exit;		/* catchup thread with left arrow key or not */
 	t_bool unlink_article;
@@ -282,6 +289,9 @@ struct t_config {
 	char select_format[LEN];		/* format string for the selection level */
 	char group_format[LEN];			/* format string for the group level */
 	char thread_format[LEN];		/* format string for the thread level */
+	char attachment_format[LEN];	/* format string for the attachment level */
+	char page_mime_format[LEN];		/* format string for the mime header in page level */
+	char page_uue_format[LEN];		/* format string for the uue header in page level */
 	char date_format[LEN];			/* format string for the date display in the page header */
 #	ifdef HAVE_UNICODE_NORMALIZATION
 		int normalization_form;
@@ -361,6 +371,9 @@ struct t_config {
 	t_bool attrib_show_signatures;
 	t_bool attrib_sigdashes;
 	t_bool attrib_signature_repost;
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		t_bool attrib_suppress_soft_hyphens;
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	t_bool attrib_tex2iso_conv;
 	t_bool attrib_thread_catchup_on_exit;
 	t_bool attrib_verbatim_handling;
