@@ -3,10 +3,10 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2021-07-23
+ *  Updated   : 2023-11-27
  *  Notes     :
  *
- * Copyright (c) 1992-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,6 +44,9 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
+#ifndef STPWATCH_H
+#	include "stpwatch.h"
+#endif /* !STPWATCH_H */
 
 /*
  * List of allowed separator chars in active file
@@ -173,7 +176,7 @@ active_add(
 
 	if (moderated[0] == '/') {
 		ptr->type = GROUP_TYPE_SAVE;
-		ptr->spooldir = my_strdup(moderated); /* TODO: Unix'ism, other OSs need transformation */
+		ptr->spooldir = my_strdup(moderated);
 	} else {
 		ptr->type = GROUP_TYPE_NEWS;
 		ptr->spooldir = spooldir;		/* another global - sigh */
@@ -227,40 +230,62 @@ parse_active_line(
 	char *moderated)
 {
 	char *p = NULL, *q = NULL, *r = NULL;
+#ifdef DEBUG
+	char *l;
+#endif /* DEBUG */
 	t_bool lineok = FALSE;
 
 	if (line[0] == '#' || line[0] == '\0')
-		return FALSE;
+		return lineok;
+
+#ifdef DEBUG
+	l = my_strdup(line);
+#endif /* DEBUG */
 
 	if (strtok(line, ACTIVE_SEP)) {		/* skip group name */
 		if ((p = strtok(NULL, ACTIVE_SEP))) {	/* group max count */
 			if ((q = strtok(NULL, ACTIVE_SEP))) {	/* group min count */
-				r = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
-				lineok = TRUE;
+				if ((r = strtok(NULL, ACTIVE_SEP)))	/* mod status or path to mailgroup */
+					lineok = TRUE;
 			}
 		}
 	}
 
-	if (!p || !q || !r || !lineok) {
+	if (!lineok) {
 #ifdef DEBUG
 		/* TODO: This also logs broken non NNTP active lines (i.e. mail.active) to NNTP */
 		if ((debug & DEBUG_NNTP) && verbose > 1)
-			debug_print_file("NNTP", "Active file corrupt - %s", line);
+			debug_print_file("NNTP", "Active file corrupt - %s", l);
+
 #endif /* DEBUG */
-		return FALSE;
+		if (!p || !q) {
+#ifdef DEBUG
+			free(l);
+#endif /* DEBUG */
+			return lineok;
+		}
 	}
 
 	*max = atoartnum(p);
 	*min = atoartnum(q);
-	strcpy(moderated, r);
 
-	return TRUE;
+	if (!lineok) { /* missing moderation flag - seen on usenet.farm */
+		strcpy(moderated, "y");	/* guess posting is fine */
+		lineok = TRUE;
+	} else
+		strcpy(moderated, r);
+
+#ifdef DEBUG
+	free(l);
+#endif /* DEBUG */
+
+	return lineok;
 }
 
 
 #ifdef NNTP_ABLE
 /*
- * Parse line from "LIST COUNTS" (RFC 6048)
+ * Parse line from "LIST COUNTS" (RFC 6048 2.2.)
  * group high low count status
  */
 static t_bool
@@ -334,9 +359,8 @@ do_read_newsrc_active_file(
 	struct t_group *grpptr;
 #ifdef NNTP_ABLE
 	t_bool need_auth = FALSE;
-	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND];
-	int index_i = 0;
-	int index_o = 0;
+	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND] = { NULL };
+	int index_i = 0, index_o = 0;
 #endif /* NNTP_ABLE */
 
 	rewind(fp);
@@ -389,6 +413,7 @@ do_read_newsrc_active_file(
 					 */
 					int i;
 					int j = index_o;
+
 					for (i = 0; i < window - 1; i++) {
 						snprintf(buf, sizeof(buf), "GROUP %s", ngnames[j]);
 #	ifdef DEBUG
@@ -415,14 +440,11 @@ do_read_newsrc_active_file(
 							snprintf(fmt, sizeof(fmt), "%%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%%ds", NNTP_GRPLEN);
 							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4) {
 #	ifdef DEBUG
-								if ((debug & DEBUG_NNTP) && verbose > 1)
+								if ((debug & DEBUG_NNTP) && verbose > 1) {
 									debug_print_file("NNTP", "Invalid response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
-#	endif /* DEBUG */
-							}
-							if (strcmp(ngname, ngnames[index_o]) != 0) {
-#	ifdef DEBUG
-								if ((debug & DEBUG_NNTP) && verbose > 1)
-									debug_print_file("NNTP", "Groupname mismatch in response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
+									if (strcmp(ngname, ngnames[index_o]) != 0)
+										debug_print_file("NNTP", "Groupname mismatch in response to \"GROUP %s\": \"%s\"", ngnames[index_o], line);
+								}
 #	endif /* DEBUG */
 							}
 							ptr = ngname;
@@ -542,7 +564,12 @@ read_newsrc_active_file(
 
 #ifdef NNTP_ABLE
 	if (need_auth) { /* delayed auth */
-		if (!authenticate(nntp_server, userid, FALSE) || do_read_newsrc_active_file(fp))
+		if (!authenticate(nntp_server, userid, FALSE))
+			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+		set_maxartnum(FALSE);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+		if (do_read_newsrc_active_file(fp))
 			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
 	}
 #endif /* NNTP_ABLE */
@@ -597,17 +624,20 @@ read_active_file(
 		wait_message(0, _(txt_reading_news_active_file));
 
 	if ((fp = open_news_active_fp()) == NULL) {
-		if (cmd_line && !batch_mode)
+		if ((cmd_line && !batch_mode) || verbose)
 			my_fputc('\n', stderr);
 
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp)
 			tin_done(EXIT_FAILURE, _(txt_cannot_retrieve), ACTIVE_FILE);
 #	ifndef NNTP_ONLY
-		else
+		else {
+			perror_message("%s", news_active_file);
 			tin_done(EXIT_FAILURE, _(txt_cannot_open_active_file), news_active_file, tin_progname);
+		}
 #	endif /* !NNTP_ONLY */
 #else
+		perror_message("%s", news_active_file);
 		tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
 	}
@@ -651,6 +681,10 @@ read_active_file(
 		 */
 		active_add(grpptr, count, max, min, moderated);
 	}
+#	if defined(DEBUG) && defined(NNTP_ABLE)
+	if ((debug & DEBUG_NNTP) && !verbose)
+		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG && NNTP_ABLE */
 
 	TIN_FCLOSE(fp);
 
@@ -729,6 +763,11 @@ read_active_counts(
 		 */
 		active_add(grpptr, count, max, min, moderated);
 	}
+#	ifdef DEBUG
+	/* log end of multiline response to get timing data */
+	if ((debug & DEBUG_NNTP) && !verbose)
+		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG */
 
 	/*
 	 * Exit if active file wasn't read correctly or is empty
@@ -884,11 +923,17 @@ read_news_active_file(
 								}
 								active_add(grpptr, count, max, min, moderated);
 							}
+#	ifdef DEBUG
+							/* log end of multiline response to get timing data */
+							if ((debug & DEBUG_NNTP) && !verbose)
+								debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG */
 						}
 					}
 					if (need_auth) { /* retry after auth is overkill here, so just auth */
 						if (!authenticate(nntp_server, userid, FALSE))
 							tin_done(EXIT_FAILURE, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS);
+						/* no set_maxartnum() here after auth as we're pipelining ... */
 					}
 				}
 				did_reconnect = FALSE;
@@ -912,7 +957,9 @@ read_news_active_file(
 	/*
 	 * finally we have a list of all groups and can set the attributes
 	 */
+	BegStopWatch();
 	assign_attributes_to_groups();
+	EndStopWatch("assign_attributes_to_groups()");
 
 	return newgrps;
 }
@@ -949,6 +996,9 @@ open_newgroups_fp(
 
 		return (nntp_command(line, OK_NEWGROUPS, NULL, 0));
 	}
+#else
+	/* silence compiler warning (unused parameter) */
+	(void) idx;
 #endif /* NNTP_ABLE */
 	return (fopen(active_times_file, "r"));
 }
@@ -1265,7 +1315,6 @@ group_flag(
 }
 
 
-/* ex actived.c functions */
 void
 create_save_active_file(
 	void)
