@@ -3,10 +3,10 @@
  *  Module    : art.c
  *  Author    : I.Lea & R.Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2022-01-28
+ *  Updated   : 2023-07-26
  *  Notes     :
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,11 +47,11 @@
 #ifndef NEWSRC_H
 #	include "newsrc.h"
 #endif /* !NEWSRC_H */
-
 #ifndef STPWATCH_H
 #	include "stpwatch.h"
 #endif /* !STPWATCH_H */
 
+
 /*
  * TODO: fixup to remove CURR_GROUP dependency in all sort funcs
  */
@@ -292,6 +292,11 @@ setup_hard_base(
 					base[grpmenu.max++] = atoartnum(ptr);
 					total++;
 				}
+#	ifdef DEBUG
+				/* log end of multiline response to get timing data */
+				if ((debug & DEBUG_NNTP) && !verbose)
+					debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+#	endif /* DEBUG */
 
 				if (tin_errno)
 					return -1;
@@ -424,16 +429,14 @@ index_group(
 	free_art_array();
 	free_msgids();
 
-	BegStopWatch("setup_hard_base()");
-
+	BegStopWatch();
 	/*
 	 * Get list of valid article numbers
 	 */
 	if (setup_hard_base(group) < 0)
 		return FALSE;
 
-	EndStopWatch();
-	PrintStopWatch();
+	EndStopWatch("setup_hard_base()");
 
 #ifdef DEBUG
 	if (debug & DEBUG_NEWSRC) {
@@ -570,22 +573,23 @@ index_group(
 	 * Create the reference tree. The msgid and ref ptrs will
 	 * be free()d now that the NovFile has been written.
 	 */
+	BegStopWatch();
 	build_references(group);
+	EndStopWatch("build_references()");
 
 	/*
 	 * Needs access to the reference tree
 	 */
+	BegStopWatch();
 	filtered = filter_articles(group);
-
-	BegStopWatch("make_threads()");
+	EndStopWatch("filter_articles()");
 
 	/*
 	 * Thread the group
 	 */
+	BegStopWatch();
 	make_threads(group, FALSE);
-
-	EndStopWatch();
-	PrintStopWatch();
+	EndStopWatch("make_threads()");
 
 	if ((changed > 0 || filtered) && !batch_mode)
 		clear_message();
@@ -616,7 +620,7 @@ find_first_unread(
 
 /*
  * Open an article for reading just the header
- * 'next' is used/updated with the next article number
+ * 'NEXT' is used/updated with the next article number
  * to optimise the number of 'HEAD' commands issued on
  * groups with holes.
  */
@@ -632,6 +636,7 @@ open_art_header(
 	int i;
 
 	if (read_news_via_nntp && CURR_GROUP.type == GROUP_TYPE_NEWS) {
+		static t_bool no_next = FALSE; /* TODO: move to t_capabilities ? */
 		/*
 		 * Don't bother requesting if we have not got there yet.
 		 * This is a big win if the group has got holes in it (ie. if 000's
@@ -649,55 +654,67 @@ open_art_header(
 		 * shall we stop on 5xx?, i.e JamNNTPd/2 1.3 responds with
 		 * "503 Access denied" instead of 480 but NEXT still works,
 		 * so tin loops over all articles without getting useful data
+		 *
+		 * usenet.farm may return ERR_GOODBYE and NEXT then also fails
+		 * we could flag that like no_next ...
 		 */
 
-		/*
-		 * HEAD failed, try to find NEXT
-		 * Should return "223 artno message-id more text...."
-		 */
-		i = new_nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf));
-		switch (i) {
-			case OK_NOTEXT:
-				*next = atoartnum(buf);		/* Set next art number */
-				break;
-
-#	ifndef BROKEN_LISTGROUP
+		if (!no_next) { /* usenet.farm doesn't do NEXT */
 			/*
-			 * might happen if LISTGROUP doesn't select group, but
-			 * we are not -DBROKEN_LISTGROUP
+			 * HEAD failed, try to find NEXT
+			 * Should return "223 artno message-id more text...."
 			 */
-			case ERR_NCING:
-				nntp_caps.broken_listgroup = TRUE;
-				snprintf(buf, sizeof(buf), "GROUP %s", groupname);
-				if (nntp_command(buf, OK_GROUP, NULL, 0) == NULL)
-					return NULL;
-				snprintf(buf, sizeof(buf), "HEAD %"T_ARTNUM_PFMT, art);
-				if ((fp = nntp_command(buf, OK_HEAD, NULL, 0)) != NULL)
-					return fp;
-				if (nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf)))
-					*next = atoartnum(buf);
-				break;
-#	endif /* !BROKEN_LISTGROUP */
+			i = new_nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf));
+			switch (i) {
+				case OK_NOTEXT:
+					*next = atoartnum(buf);		/* Set next art number */
+					break;
 
-			default:
-				/*
-				 * TODO: abort loop over all arts on ERR_NONEXT
-				 */
 #	ifndef BROKEN_LISTGROUP
 				/*
-				 * to avoid out of sync responses
-				 * (listgroup seems to work, but didn't select new group,
-				 *  so xover seems to work but returns old data)
-				 * we set listgroup_broken = TRUE; once we saw a
-				 * ERR_NOARTIG / ERR_NONEXT or the like - even if
-				 * ERR_NOARTIG may occur on servers where listgroup
-				 * isn't broken...
+				 * might happen if LISTGROUP doesn't select group, but
+				 * we are not -DBROKEN_LISTGROUP
 				 */
-				nntp_caps.broken_listgroup = TRUE;
+				case ERR_NCING:
+					nntp_caps.broken_listgroup = TRUE;
+					snprintf(buf, sizeof(buf), "GROUP %s", groupname);
+					if (nntp_command(buf, OK_GROUP, NULL, 0) == NULL)
+						return NULL;
+					snprintf(buf, sizeof(buf), "HEAD %"T_ARTNUM_PFMT, art);
+					if ((fp = nntp_command(buf, OK_HEAD, NULL, 0)) != NULL)
+						return fp;
+					if (nntp_command("NEXT", OK_NOTEXT, buf, sizeof(buf)))
+						*next = atoartnum(buf);
+					break;
 #	endif /* !BROKEN_LISTGROUP */
-				break;
-		}
 
+				case ERR_COMMAND:	/* TODO: abort loop over all arts */
+					no_next = TRUE;
+#	ifdef DEBUG
+					if ((debug & DEBUG_NNTP) && verbose > 1)
+						debug_print_file("NNTP", "!!! NEXT disabled after %d response", ERR_COMMAND);
+#	endif /* DEBUG */
+					break;
+
+				default:
+					/*
+					 * TODO: abort loop over all arts on ERR_NONEXT
+					 */
+#	ifndef BROKEN_LISTGROUP
+					/*
+					 * to avoid out of sync responses
+					 * (listgroup seems to work, but didn't select new group,
+					 *  so xover seems to work but returns old data)
+					 * we set listgroup_broken = TRUE; once we saw a
+					 * ERR_NOARTIG / ERR_NONEXT or the like - even if
+					 * ERR_NOARTIG may occur on servers where listgroup
+					 * isn't broken...
+					 */
+					nntp_caps.broken_listgroup = TRUE;
+#	endif /* !BROKEN_LISTGROUP */
+					break;
+			}
+		}
 		return NULL;
 	}
 #endif /* NNTP_ABLE */
@@ -754,6 +771,7 @@ read_art_headers(
 
 	snprintf(group_msg, sizeof(group_msg), _(txt_group), cCOLS - MIN(cCOLS - 1, strwidth(_(txt_group))) + 2 - 3, group->name);
 
+	/* TODO: add progress meter? HEAD/NEXT is slow */
 	for (i = 0; i < grpmenu.max; i++) {	/* for each article number */
 		art = base[i];
 
@@ -1093,7 +1111,7 @@ global_get_multiparts(
 {
 	int i, part_index, part_cnt = 0;
 	MultiPartInfo tmp, tmp2;
-	MultiPartInfo *info = NULL;
+	MultiPartInfo *info;
 
 	/* entry assertions */
 	assert(((void) "Invalid index", 0 <= aindex && aindex < top_art));
@@ -1579,7 +1597,7 @@ build_range_list(
 			 * if we reached top_art all arts have path
 			 * so we use max
 			 */
-			curr->start = i == top_art ? max : arts[i--].artnum;
+			curr->start = ((i == top_art) ? max : arts[i--].artnum);
 		} else {
 			for (; i < top_art && !arts[i].path; i++)
 				;
@@ -1750,7 +1768,7 @@ get_path_header(
 		if (min == max)
 			snprintf(cmd, sizeof(cmd), "%s Path %"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, min);
 		else
-			snprintf(cmd, sizeof(cmd), "%s Path %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, min, max);
+			snprintf(cmd, sizeof(cmd), "%s Path %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, min, MAX(min, max));
 		fp = nntp_command(cmd, nntp_caps.hdr_cmd[0] == 'X' ? OK_XHDR : OK_HDR, NULL, 0);
 		if (!nntp_caps.hdr && fp)
 			nntp_caps.hdr = TRUE;
@@ -1758,7 +1776,7 @@ get_path_header(
 		if (min == max)
 			snprintf(cmd, sizeof(cmd), "XPAT Path %"T_ARTNUM_PFMT" *", min);
 		else
-			snprintf(cmd, sizeof(cmd), "XPAT Path %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT" *", min, max);
+			snprintf(cmd, sizeof(cmd), "XPAT Path %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT" *", min, MAX(min, max));
 		fp = nntp_command(cmd, OK_XPAT, NULL, 0);
 	}
 
@@ -1787,6 +1805,11 @@ get_path_header(
 			if (++artnum % MODULO_COUNT_NUM == 0)
 				show_progress(prep_msg, artnum - min, max - min);
 		}
+#	ifdef DEBUG
+		/* log end of multiline response to get timing data */
+		if ((debug & DEBUG_NNTP) && !verbose)
+			debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+#	endif /* DEBUG */
 		free(prep_msg);
 		return supported;
 	}
@@ -1814,11 +1837,11 @@ get_path_header(
  * If 'local' is set then always open local overview cache in preference to
  * using NNTP XOVER
  *
- * Format (mandatory as far as line count [RFC2980]):
+ * Format (mandatory as far as line count [RFC 2980]):
  *	1. article number (ie. 183)                [mandatory]
  *	2. Subject: line  (ie. Which newsreader?)  [mandatory]
  *	3. From: line     (ie. iain@ecrc.de)       [mandatory]
- *	4. Date: line     (rfc822 format)          [mandatory]
+ *	4. Date: line     (RFC 822 format)         [mandatory]
  *	5. MessageID:     (ie. <123@example.net>)  [mandatory]
  *	6. References:    (ie. <message-id> ....)  [optional]
  *	7. Byte count     (Skipped - not used)     [mandatory]
@@ -2233,7 +2256,7 @@ read_overview(
 					/* if we're lucky we've Path in NOV */
 					/*
 					 * if reading locally cached overview data try
-					 * path regardless of the server OVERVIEW.FMT
+					 * path regardless of the servers OVERVIEW.FMT
 					 */
 					if (local || !strcasecmp(ofmt[count].name, "Path:")) {
 						if ((q = parse_header(ptr, "Path", FALSE, FALSE, FALSE)) != NULL) {
@@ -2268,6 +2291,11 @@ read_overview(
 
 		top_art++;				/* Basically this statement commits the article */
 	}
+#	ifdef DEBUG
+	/* log end of multiline response to get timing data */
+	if ((debug & DEBUG_NNTP) && !verbose)
+		debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+#	endif /* DEBUG */
 
 	free(group_msg);
 	TIN_FCLOSE(fp);
@@ -2353,6 +2381,11 @@ read_overview(
 					if (artnum % (MODULO_COUNT_NUM * 20) == 0)
 						show_progress(group_msg, artnum - min, max - min);
 				}
+#	ifdef DEBUG
+				/* log end of multiline response to get timing data */
+				if ((debug & DEBUG_NNTP) && !verbose)
+					debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+#	endif /* DEBUG */
 			}
 			free(group_msg);
 		}
@@ -2417,7 +2450,7 @@ read_overview(
  *	1. article number (ie. 183)                [mandatory]
  *	2. Subject: line  (ie. Which newsreader?)  [mandatory]
  *	3. From: line     (ie. iain@ecrc.de)       [mandatory]
- *	4. Date: line     (rfc822 format)          [mandatory]
+ *	4. Date: line     (RFC 822 format)         [mandatory]
  *	5. MessageID:     (ie. <123@example.net>)  [mandatory]
  *	6. References:    (ie. <message-id> ....)  [optional]
  *	7. Byte count     (Skipped - not used)     [mandatory]
@@ -2565,11 +2598,11 @@ write_overview(
 			show_progress(_("Writing overview cache..."), i, top_art);
 	}
 #ifdef HAVE_FCHMOD
-	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH));
-	/*
-	 * TODO:
-	 * add code for !HAVE_FCHMOD && HAVE_CHMOD
-	 */
+	fchmod(fileno(fp), (mode_t) (S_IWUSR|S_IRUGO));
+#else
+#	ifdef HAVE_CHMOD
+		chmod(find_nov_file(group,R_OK), (mode_t) (S_IWUSR|S_IRUGO));
+#	endif /* HAVE_CHMOD */
 #endif /* HAVE_FCHMOD */
 	fclose(fp);
 }
@@ -3083,11 +3116,13 @@ score_comp_base(
 
 		if (comp_func)
 			return (*comp_func)(s1, s2);
+
 		return 0;
 	}
 
 	if (CURR_GROUP.attribute->sort_threads_type == SORT_THREADS_BY_SCORE_ASCEND)
 		return a > b ? 1 : -1;
+
 	return a < b ? 1 : -1;
 }
 
