@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2022-03-04
+ *  Updated   : 2023-10-29
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -183,6 +183,9 @@ read_config_file(
 			if (match_boolean(buf, "ask_for_metamail=", &tinrc.ask_for_metamail))
 				break;
 
+			if (match_string(buf, "attachment_format=", tinrc.attachment_format, sizeof(tinrc.attachment_format)))
+				break;
+
 			if (match_integer(buf, "auto_cc_bcc=", &tinrc.auto_cc_bcc, AUTO_CC_BCC))
 				break;
 
@@ -567,6 +570,12 @@ read_config_file(
 			break;
 
 		case 'p':
+			if (match_string(buf, "page_mime_format=", tinrc.page_mime_format, sizeof(tinrc.page_mime_format)))
+				break;
+
+			if (match_string(buf, "page_uue_format=", tinrc.page_uue_format, sizeof(tinrc.page_uue_format)))
+				break;
+
 			if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, &tinrc.post_mime_encoding))
 				break;
 
@@ -751,6 +760,11 @@ read_config_file(
 			if (match_string(buf, "strip_was_regex=", tinrc.strip_was_regex, sizeof(tinrc.strip_was_regex)))
 				break;
 
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			if (match_boolean(buf, "suppress_soft_hyphens=", &tinrc.suppress_soft_hyphens))
+				break;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
 			break;
 
 		case 't':
@@ -772,6 +786,11 @@ read_config_file(
 			if (match_boolean(buf, "thread_catchup_on_exit=", &tinrc.thread_catchup_on_exit))
 				break;
 
+#ifdef NNTPS_ABLE
+			if (match_string(buf, "tls_ca_cert_file=", tinrc.tls_ca_cert_file, sizeof(tinrc.tls_ca_cert_file)))
+				break;
+#endif /* NNTPS_ABLE */
+
 #if defined(HAVE_ICONV_OPEN_TRANSLIT) && defined(CHARSET_CONVERSION)
 			if (match_boolean(buf, "translit=", &tinrc.translit))
 				break;
@@ -910,6 +929,8 @@ read_config_file(
 		tinrc.post_8bit_header = FALSE;
 
 	/* set defaults if blank */
+	if (!*tinrc.attachment_format)
+		STRCPY(tinrc.attachment_format, DEFAULT_ATTACHMENT_FORMAT);
 	if (!*tinrc.editor_format)
 		STRCPY(tinrc.editor_format, TIN_EDITOR_FMT);
 	if (!*tinrc.select_format)
@@ -928,6 +949,10 @@ read_config_file(
 		STRCPY(tinrc.mm_charset, get_val("MM_CHARSET", MM_CHARSET));
 	strcpy(tinrc.mm_local_charset, tinrc.mm_charset);
 #endif /* !CHARSET_CONVERSION */
+	if (!*tinrc.page_mime_format)
+		STRCPY(tinrc.page_mime_format, DEFAULT_PAGE_MIME_FORMAT);
+	if (!*tinrc.page_uue_format)
+		STRCPY(tinrc.page_uue_format, DEFAULT_PAGE_UUE_FORMAT);
 
 	return TRUE;
 }
@@ -1159,6 +1184,11 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_show_signatures.tinrc));
 	fprintf(fp, "show_signatures=%s\n\n", print_boolean(tinrc.show_signatures));
 
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	fprintf(fp, "%s", _(txt_suppress_soft_hyphens.tinrc));
+	fprintf(fp, "suppress_soft_hyphens=%s\n\n", print_boolean(tinrc.suppress_soft_hyphens));
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
 	fprintf(fp, "%s", _(txt_tex2iso_conv.tinrc));
 	fprintf(fp, "tex2iso_conv=%s\n\n", print_boolean(tinrc.tex2iso_conv));
 
@@ -1440,6 +1470,15 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_thread_format.tinrc));
 	fprintf(fp, "thread_format=%s\n\n", tinrc.thread_format);
 
+	fprintf(fp, "%s", _(txt_attachment_format.tinrc));
+	fprintf(fp, "attachment_format=%s\n\n", tinrc.attachment_format);
+
+	fprintf(fp, "%s", _(txt_page_mime_format.tinrc));
+	fprintf(fp, "page_mime_format=%s\n\n", tinrc.page_mime_format);
+
+	fprintf(fp, "%s", _(txt_page_uue_format.tinrc));
+	fprintf(fp, "page_uue_format=%s\n\n", tinrc.page_uue_format);
+
 	fprintf(fp, "%s", _(txt_date_format.tinrc));
 	fprintf(fp, "date_format=%s\n\n", tinrc.date_format);
 
@@ -1456,6 +1495,11 @@ write_config_file(
 	fprintf(fp, "render_bidi=%s\n\n", print_boolean(tinrc.render_bidi));
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
 
+#ifdef NNTPS_ABLE
+	fprintf(fp, "%s", _(txt_tls_ca_cert_file.tinrc));
+	fprintf(fp, "tls_ca_cert_file=%s\n\n", tinrc.tls_ca_cert_file);
+#endif /* NNTPS_ABLE */
+
 	fprintf(fp, "%s", _(txt_tinrc_filter));
 	fprintf(fp, "default_filter_kill_header=%d\n", tinrc.default_filter_kill_header);
 	fprintf(fp, "default_filter_kill_global=%s\n", print_boolean(tinrc.default_filter_kill_global));
@@ -1590,7 +1634,7 @@ match_color(
 
 /*
  * If pat matches the start of line, convert rest of line to an integer, dst
- * If maxval is set, constrain value to 0 <= dst <= maxlen and return TRUE.
+ * If maxval is set, constrain value to 0 <= dst <= maxval and return TRUE.
  * If no match is made, return FALSE.
  */
 t_bool
@@ -2208,17 +2252,18 @@ static void
 write_server_config(
 	void)
 {
+	DIR *dirp;
 	FILE *fp;
 	char *file_tmp;
 	char file[PATH_LEN];
 	char timestring[30];
 	char serverdir[PATH_LEN];
 	int i;
-	struct stat statbuf;
 
 	if (read_saved_news)
 		/* don't update server files while reading locally stored articles */
 		return;
+
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && nntp_tcp_port != IPPORT_NNTP)
 		snprintf(file, sizeof(file), "%s:%u", nntp_server, nntp_tcp_port);
@@ -2233,11 +2278,23 @@ write_server_config(
 	if ((no_write || post_article_and_exit || post_postponed_and_exit) && file_size(file) != -1L)
 		return;
 
-	if (stat(serverdir, &statbuf) == -1) {
-		if (my_mkdir(serverdir, (mode_t) (S_IRWXU)) == -1)
-			/* Can't create directory TODO: Add error handling */
-			return;
-	}
+	errno = 0;
+	if (!(dirp = opendir(serverdir))) {
+		switch (errno) {
+			case ENOENT:
+				if (my_mkdir(serverdir, (mode_t) (S_IRWXU)) == -1) {
+					/* Can't create directory TODO: Add error handling */
+					;
+				}
+				break;
+
+			default:
+				wait_message(2, "write_server_config(%s)", strerror(errno));
+				break;
+		}
+		return;
+	} else
+		CLOSEDIR(dirp);
 
 	/* generate tmp-filename */
 	file_tmp = get_tmpfilename(file);
