@@ -3,10 +3,10 @@
  *  Module    : feed.c
  *  Author    : I. Lea
  *  Created   : 1991-08-31
- *  Updated   : 2022-08-26
+ *  Updated   : 2023-10-16
  *  Notes     : provides same interface to mail,pipe,print,save & repost commands
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -161,7 +161,7 @@ expand_feed_filename(
 	 * If no path exists or the above failed in some way, use sensible defaults
 	 * Put the generic path into 'outpath'
 	 */
-	if ((ret == 0) || !(strrchr(outpath, DIRSEP))) {
+	if ((ret == 0) || !(strrchr(outpath, '/'))) {
 		char buf[PATH_LEN];
 
 		if (!strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE))
@@ -355,7 +355,6 @@ print_save_summary(
 	t_function type,
 	int fed)
 {
-	const char *first, *last;
 	char buf[LEN];
 	char what[LEN];
 
@@ -382,18 +381,15 @@ print_save_summary(
 			break;
 	}
 
-	first = (save[0].mailbox) ? save[0].path : save[0].file;
-	last = (save[num_save - 1].mailbox) ? save[num_save - 1].path : save[num_save - 1].file;
-
 	/*
 	 * We report the range of saved-to files for regular saves of > 1 articles
 	 */
 	if (num_save == 1 || save[0].mailbox)
 		snprintf(buf, sizeof(buf), _(txt_saved_to),
-			what, (save[0].mailbox ? _(txt_mailbox) : ""), first);
+			what, (save[0].mailbox ? _(txt_mailbox) : ""), save[0].path);
 	else
 		snprintf(buf, sizeof(buf), _(txt_saved_to_range),
-			what, first, last);
+			what, save[0].file, save[num_save - 1].file);
 
 	wait_message((tinrc.beginner_level) ? 4 : 2, buf);
 }
@@ -529,6 +525,7 @@ feed_article(
 
 	if (!use_current)
 		art_close(openartptr);
+
 	return ok;
 }
 
@@ -704,7 +701,7 @@ feed_articles(
 					if (function != FEED_AUTOSAVE && (pproc_func = get_post_proc_type()) == GLOBAL_ABORT)
 						return -1;
 				}
-				if (!create_path(outpath))
+				if (create_path(outpath) != 0)
 					return -1;
 			}
 			break;
@@ -774,7 +771,7 @@ feed_articles(
 
 	/*
 	 * Performance hack - If we feed a single art from the pager then we can
-	 * re-use the currently open article
+	 * reuse the currently open article
 	 * Also no need to fetch articles just to mark them (un)read
 	 */
 	if (feed_mark_function || (level == PAGE_LEVEL && (feed_type == FEED_ARTICLE || feed_type == FEED_THREAD))) {
@@ -865,9 +862,8 @@ feed_articles(
 					counter.max++;
 				}
 
-				if (tinrc.wildcard) {
+				if (tinrc.wildcard)
 					regex_cache_destroy(&cache);
-				}
 			}
 
 			/* I think we nest like this to preserve any 'ordering' of the arts */
@@ -984,7 +980,7 @@ got_epipe_while_piping:
 			curr_line = saved_curr_line;
 
 		if (redraw_screen)
-			draw_page(group->name, 0);
+			draw_page(0);
 		else {
 			if (function == FEED_PIPE)
 				clear_message();
