@@ -3,10 +3,10 @@
  *  Module    : filter.c
  *  Author    : I. Lea
  *  Created   : 1992-12-28
- *  Updated   : 2022-02-19
+ *  Updated   : 2023-11-22
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -102,7 +102,7 @@ static struct t_filter_comment *add_filter_comment(struct t_filter_comment *ptr,
 static struct t_filter_comment *free_filter_comment(struct t_filter_comment *ptr);
 static struct t_filter_comment *copy_filter_comment(struct t_filter_comment *from, struct t_filter_comment *to);
 static t_bool add_filter_rule(struct t_group *group, struct t_article *art, struct t_filter_rule *rule, t_bool quick_filter_rule);
-static int test_regex(const char *string, char *regex, t_bool nocase, struct regex_cache *cache);
+static int test_match(const char *string, char *regex, t_bool nocase, t_bool use_regex, struct regex_cache *cache);
 static void expand_filter_array(struct t_filters *ptr);
 static void fmt_filter_menu_prompt(char *dest, size_t dest_len, const char *fmt_str, int len, const char *text);
 static void free_filter_item(struct t_filter *ptr);
@@ -110,7 +110,7 @@ static void print_filter_menu(void);
 static void set_filter(struct t_filter *ptr);
 static void write_filter_array(FILE *fp, struct t_filters *ptr);
 #if 0 /* currently unused */
-	static FILE *open_xhdr_fp(char *header, long min, long max);
+	static FILE *open_xhdr_fp(char *header, t_artnum min, t_artnum max);
 #endif /* 0 */
 
 
@@ -200,29 +200,30 @@ expand_filter_array(
  * In case of error prints an error message and returns -1
  */
 static int
-test_regex(
+test_match(
 	const char *string,
 	char *regex,
 	t_bool nocase,
+	t_bool use_regex,
 	struct regex_cache *cache)
 {
-	int regex_errpos;
+	int error;
 
-	if (!tinrc.wildcard) {
+	if (!use_regex) {
 		if (wildmat(string, regex, nocase))
 			return 1;
 	} else {
 		if (!cache->re)
-			compile_regex(regex, cache, (nocase ? PCRE_CASELESS : 0));
+			compile_regex(regex, cache, (nocase ? REGEX_CASELESS : 0));
 		if (cache->re) {
-			regex_errpos = pcre_exec(cache->re, cache->extra, string, (int) strlen(string), 0, 0, NULL, 0);
-			if (regex_errpos >= 0)
+			error = match_regex_ex(string, (int) strlen(string), 0, 0, cache);
+			if (error >= 0)
 				return 1;
-			else if (regex_errpos != PCRE_ERROR_NOMATCH) { /* also exclude PCRE_ERROR_BADUTF8 ? */
-				error_message(2, _(txt_pcre_error_num), regex_errpos);
+			else if (error != REGEX_ERROR_NOMATCH) { /* also exclude BADUTF8 ? */
+				error_message(2, _(txt_pcre_error_num), error);
 #ifdef DEBUG
 				if (debug & DEBUG_FILTER) {
-					debug_print_file("FILTER", _(txt_pcre_error_num), regex_errpos);
+					debug_print_file("FILTER", _(txt_pcre_error_num), error);
 					debug_print_file("FILTER", "\t regex: %s", regex);
 					debug_print_file("FILTER", "\tstring: %s", string);
 				}
@@ -259,7 +260,6 @@ set_filter(
 		ptr->xref = NULL;
 		ptr->path = NULL;
 		ptr->time = (time_t) 0;
-		ptr->next = (struct t_filter *) 0;
 	}
 }
 
@@ -369,7 +369,7 @@ read_filter_file(
 			case 'c':
 				if (match_integer(buf + 1, "ase=", &icase, 1)) {
 					if (ptr && !expired_time)
-						ptr[i].icase = (unsigned) icase;
+						ptr[i].icase = (t_bool) icase;
 
 					break;
 				}
@@ -703,8 +703,7 @@ write_filter_file(
 		return;
 	}
 
-	/* TODO: -> lang.c */
-	fprintf(fp, "# Filter file V%s for the TIN newsreader\n#\n", FILTER_VERSION);
+	fprintf(fp, txt_filter_file_version, FILTER_VERSION);
 	fprintf(fp, "%s", _(txt_filter_file));
 
 	fflush(fp);
@@ -797,7 +796,7 @@ write_filter_array(
 
 		fprintf(fp, "group=%s\n", (ptr->filter[i].scope != NULL ? ptr->filter[i].scope : "*"));
 
-		fprintf(fp, "case=%u\n", ptr->filter[i].icase);
+		fprintf(fp, "case=%u\n", ptr->filter[i].icase ? 1U : 0U);
 
 		if (ptr->filter[i].score == tinrc.score_kill)
 			fprintf(fp, "score=kill\n");
@@ -881,6 +880,7 @@ write_filter_array(
 
 		if (ptr->filter[i].time) {
 			char timestring[25];
+
 			if (my_strftime(timestring, sizeof(timestring) - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(ptr->filter[i].time))))
 				fprintf(fp, "time=%lu (%s)\n", (unsigned long int) ptr->filter[i].time, timestring);
 		}
@@ -1074,8 +1074,8 @@ filter_menu(
 	char buf[LEN];
 	char keyedit[MAXKEYLEN], keyquit[MAXKEYLEN], keysave[MAXKEYLEN];
 	char text_time[PATH_LEN];
-	char double_time[PATH_LEN];
-	char quat_time[PATH_LEN];
+	char *double_time;
+	char *quat_time;
 	int i, len, clen, flen;
 	struct t_filter_rule rule;
 	t_bool proceed;
@@ -1167,7 +1167,7 @@ filter_menu(
 		comment_line[0] = '\0';
 	}
 	if (!proceed) {
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1176,7 +1176,7 @@ filter_menu(
 	 */
 	show_menu_help(_(txt_help_filter_text));
 	if (!prompt_menu_string(INDEX_TOP + 2, ptr_filter_text, rule.text)) {
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1195,7 +1195,7 @@ filter_menu(
 		free(list);
 
 		if (i == -1) {
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return FALSE;
 		}
 
@@ -1233,7 +1233,7 @@ filter_menu(
 		free(list);
 
 		if (i == -1) {
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return FALSE;
 		} else
 			rule.subj_ok = (i == 0);
@@ -1253,7 +1253,7 @@ filter_menu(
 		free(list);
 
 		if (i == -1) {
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return FALSE;
 		} else
 			rule.from_ok = rule.subj_ok ? (i != 0) : (i == 0);
@@ -1277,7 +1277,7 @@ filter_menu(
 		free(list);
 
 		if (i == -1) {
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return FALSE;
 		} else {
 			switch ((rule.subj_ok || rule.from_ok) ? i : i + 1) {
@@ -1319,7 +1319,7 @@ filter_menu(
 	buf[0] = '\0';
 
 	if (!prompt_menu_string(INDEX_TOP + 9, ptr_filter_lines, buf)) {
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1358,7 +1358,7 @@ filter_menu(
 
 	buf[0] = '\0';
 	if (!prompt_menu_string(INDEX_TOP + 10, text_score, buf)) {
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1375,7 +1375,7 @@ filter_menu(
 	}
 
 	if (!rule.score) { /* ignore 0 scores */
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1394,8 +1394,10 @@ filter_menu(
 	/*
 	 * Expire time
 	 */
-	snprintf(double_time, sizeof(double_time), "2x %s", text_time);
-	snprintf(quat_time, sizeof(quat_time), "4x %s", text_time);
+	double_time = my_malloc(strlen(text_time) + 4); /* "2x " prefix */
+	quat_time = my_malloc(strlen(text_time) + 4); /* "4x " prefix*/
+	sprintf(double_time, "2x %s", text_time);
+	sprintf(quat_time, "4x %s", text_time);
 	list = my_malloc(sizeof(char *) * 4);
 	list[0] = (char *) _(txt_unlimited_time);
 	list[1] = text_time;
@@ -1403,9 +1405,11 @@ filter_menu(
 	list[3] = quat_time;
 	i = get_choice(INDEX_TOP + 11, _(txt_help_filter_time), ptr_filter_time, list, 4);
 	free(list);
+	free(double_time);
+	free(quat_time);
 
 	if (i == -1) {
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1439,11 +1443,11 @@ filter_menu(
 		free(list);
 
 		if (i == -1) {
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return FALSE;
 		}
 	} else {
-		rule.comment = free_filter_comment(rule.comment);
+		free_filter_comment(rule.comment);
 		return FALSE;
 	}
 
@@ -1465,7 +1469,7 @@ filter_menu(
 
 		case GLOBAL_QUIT:
 		case GLOBAL_ABORT:
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return FALSE;
 			/* keep lint quiet: */
 			/* FALLTHROUGH */
@@ -1475,7 +1479,7 @@ filter_menu(
 			 * Add the filter rule and save it to the filter file
 			 */
 			ret = add_filter_rule(group, art, &rule, FALSE);
-			rule.comment = free_filter_comment(rule.comment);
+			free_filter_comment(rule.comment);
 			return ret;
 			/* keep lint quiet: */
 			/* FALLTHROUGH */
@@ -1500,9 +1504,9 @@ quick_filter(
 {
 	char *scope;
 	char txt[LEN];
-	int header, expire, icase;
+	int header, expire;
 	struct t_filter_rule rule;
-	t_bool ret;
+	t_bool icase, ret;
 
 	if (type == GLOBAL_QUICK_FILTER_KILL) {
 		header = group->attribute->quick_kill_header;
@@ -1520,7 +1524,7 @@ quick_filter(
 
 #ifdef DEBUG
 	if (debug & DEBUG_FILTER)
-		error_message(2, "%s header=[%d] scope=[%s] expire=[%s] case=[%d]", (type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header, BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase);
+		error_message(2, "%s header=[%d] scope=[%s] expire=[%s] case=[%c]", (type == GLOBAL_QUICK_FILTER_KILL) ? "KILL" : "SELECT", header, BlankIfNull(scope), txt_onoff[expire != FALSE ? 1 : 0], icase ? "I" : "C");
 #endif /* DEBUG */
 
 	/*
@@ -1552,7 +1556,7 @@ quick_filter(
 	rule.score = (type == GLOBAL_QUICK_FILTER_KILL) ? tinrc.score_kill : tinrc.score_select;
 
 	ret = add_filter_rule(group, art, &rule, TRUE);
-	rule.comment = free_filter_comment(rule.comment);
+	free_filter_comment(rule.comment);
 	return ret;
 }
 
@@ -1800,7 +1804,7 @@ add_filter_rule(
 	if (filtered) {
 #ifdef DEBUG
 		if (debug & DEBUG_FILTER)
-			wait_message(2, "inscope=[%s] scope=[%s] case=[%d] subj=[%s] from=[%s] msgid=[%s] fullref=[%u] line=[%d %d] time=[%lu]", bool_unparse(ptr[i].inscope), BlankIfNull(rule->scope), ptr[i].icase, BlankIfNull(ptr[i].subj), BlankIfNull(ptr[i].from), BlankIfNull(ptr[i].msgid), ptr[i].fullref, ptr[i].lines_cmp, ptr[i].lines_num, (unsigned long int) ptr[i].time);
+			wait_message(2, "inscope=[%s] scope=[%s] case=[%c] subj=[%s] from=[%s] msgid=[%s] fullref=[%u] line=[%d %d] time=[%lu]", bool_unparse(ptr[i].inscope), BlankIfNull(rule->scope), ptr[i].icase ? "I" : "C", BlankIfNull(ptr[i].subj), BlankIfNull(ptr[i].from), BlankIfNull(ptr[i].msgid), ptr[i].fullref, ptr[i].lines_cmp, ptr[i].lines_num, (unsigned long int) ptr[i].time);
 #endif /* DEBUG */
 		write_filter_file(filter_file);
 	}
@@ -1854,6 +1858,7 @@ filter_articles(
 	struct regex_cache *regex_cache_path = NULL;
 	t_bool filtered = FALSE;
 	t_bool error = FALSE;
+	t_bool use_regex = tinrc.wildcard;
 
 	/*
 	 * check if there are any global filter rules
@@ -1878,7 +1883,7 @@ filter_articles(
 	 * set up cache tables for all types of filter rules
 	 * (only for regexp matching)
 	 */
-	if (tinrc.wildcard) {
+	if (use_regex) {
 		size_t msiz;
 
 		msiz = sizeof(struct regex_cache) * (size_t) num;
@@ -1888,25 +1893,20 @@ filter_articles(
 		regex_cache_xref = my_malloc(msiz);
 		regex_cache_path = my_malloc(msiz);
 		for (j = 0; j < num; j++) {
-			regex_cache_subj[j].re = NULL;
-			regex_cache_subj[j].extra = NULL;
-			regex_cache_from[j].re = NULL;
-			regex_cache_from[j].extra = NULL;
-			regex_cache_msgid[j].re = NULL;
-			regex_cache_msgid[j].extra = NULL;
-			regex_cache_xref[j].re = NULL;
-			regex_cache_xref[j].extra = NULL;
-			regex_cache_path[j].re = NULL;
-			regex_cache_path[j].extra = NULL;
+			regex_cache_init(&regex_cache_subj[j]);
+			regex_cache_init(&regex_cache_from[j]);
+			regex_cache_init(&regex_cache_msgid[j]);
+			regex_cache_init(&regex_cache_xref[j]);
+			regex_cache_init(&regex_cache_path[j]);
 		}
 	}
 
 	/*
 	 * loop through all arts applying global & local filtering rules
 	 */
-	for (i = 0; (i < top_art) && !error; i++) {
+	for (i = 0; i < top_art && !error; i++) {
 #ifdef HAVE_SELECT
-		if (wait_for_input())   /* allow abort */
+		if (wait_for_input()) /* allow abort */
 			break; /* to free the mem */
 #endif /* HAVE_SELECT */
 
@@ -1921,7 +1921,7 @@ filter_articles(
 				 * Filter on Subject: line
 				 */
 				if (ptr[j].subj != NULL) {
-					switch (test_regex(arts[i].subject, ptr[j].subj, ptr[j].icase, &regex_cache_subj[j])) {
+					switch (test_match(arts[i].subject, ptr[j].subj, ptr[j].icase, use_regex, &regex_cache_subj[j])) {
 						case 1:
 							SET_FILTER(group, i, j);
 							break;
@@ -1944,7 +1944,7 @@ filter_articles(
 					else
 						STRCPY(buf, arts[i].from);
 
-					switch (test_regex(buf, ptr[j].from, ptr[j].icase, &regex_cache_from[j])) {
+					switch (test_match(buf, ptr[j].from, ptr[j].icase, use_regex, &regex_cache_from[j])) {
 						case 1:
 							SET_FILTER(group, i, j);
 							break;
@@ -2007,8 +2007,8 @@ filter_articles(
 							break;
 					}
 
-					if ((x = test_regex(myrefs, ptr[j].msgid, FALSE, &regex_cache_msgid[j])) == 0) /* no match */
-						x = test_regex(mymsgid, ptr[j].msgid, FALSE, &regex_cache_msgid[j]);
+					if ((x = test_match(myrefs, ptr[j].msgid, FALSE, use_regex, &regex_cache_msgid[j])) == 0) /* no match */
+						x = test_match(mymsgid, ptr[j].msgid, FALSE, use_regex, &regex_cache_msgid[j]);
 
 					switch (x) {
 						case 1:
@@ -2105,9 +2105,9 @@ filter_articles(
 						}
 #ifdef DEBUG
 						else { /* server name missing in overview, i.e. colobus 2.1 */
-							if (debug & DEBUG_FILTER) { /* TODO: lang.c, _()? */
-								debug_print_file("FILTER", "Malformed overview entry: servername missing.");
-								debug_print_file("FILTER", "\t Xref: %s", arts[i].xref);
+							if (debug & DEBUG_FILTER) {
+								debug_print_file("FILTER", _(txt_filter_error_overview_no_servername));
+								debug_print_file("FILTER", _(txt_filter_error_overview_xref), arts[i].xref);
 							}
 						}
 #endif /* DEBUG */
@@ -2119,7 +2119,7 @@ filter_articles(
 									*e++ = ',';
 									skip = TRUE;
 								}
-								if (*s != ':' && !isspace((int) *s) && !skip)
+								if (!skip && *s != ':' && !isspace((int) *s))
 									*e++ = *s;
 								if (isspace((int) *s))
 									skip = FALSE;
@@ -2128,14 +2128,14 @@ filter_articles(
 							*--e = '\0';
 						} else {
 #ifdef DEBUG
-							if (debug & DEBUG_FILTER) /* TODO: lang.c, _()? */
-								debug_print_file("FILTER", "Skipping Xref filter");
+							if (debug & DEBUG_FILTER)
+								debug_print_file("FILTER", _(txt_filter_error_skipping_xref_filter));
 #endif /* DEBUG */
 							error = TRUE;
 							break;
 						}
 
-						switch (test_regex(k, ptr[j].xref, ptr[j].icase, &regex_cache_xref[j])) {
+						switch (test_match(k, ptr[j].xref, ptr[j].icase, use_regex, &regex_cache_xref[j])) {
 							case 1:
 								SET_FILTER(group, i, j);
 								break;
@@ -2156,7 +2156,7 @@ filter_articles(
 				 */
 				if (arts[i].path && *arts[i].path) {
 					if (ptr[j].path != NULL) {
-						switch (test_regex(arts[i].path, ptr[j].path, ptr[j].icase, &regex_cache_path[j])) {
+						switch (test_match(arts[i].path, ptr[j].path, ptr[j].icase, use_regex, &regex_cache_path[j])) {
 							case 1:
 								SET_FILTER(group, i, j);
 								break;
@@ -2172,27 +2172,23 @@ filter_articles(
 				}
 			}
 		}
-		/*
-		 * if (i % (MODULO_COUNT_NUM * 20) == 0)
-		 *    show_progress("Filter", i, top_art);
-		 */
+#if 0
+		/* useful in very high volume groups, usually it just flashes by */
+		if (i % (MODULO_COUNT_NUM * 20) == 0)
+			show_progress("Filtering", i, top_art);
+#endif /* 0 */
 	}
 
 	/*
 	 * throw away the contents of all regex_caches
 	 */
-	if (tinrc.wildcard) {
+	if (use_regex) {
 		for (j = 0; j < num; j++) {
-			FreeIfNeeded(regex_cache_subj[j].re);
-			FreeIfNeeded(regex_cache_subj[j].extra);
-			FreeIfNeeded(regex_cache_from[j].re);
-			FreeIfNeeded(regex_cache_from[j].extra);
-			FreeIfNeeded(regex_cache_msgid[j].re);
-			FreeIfNeeded(regex_cache_msgid[j].extra);
-			FreeIfNeeded(regex_cache_xref[j].re);
-			FreeIfNeeded(regex_cache_xref[j].extra);
-			FreeIfNeeded(regex_cache_path[j].re);
-			FreeIfNeeded(regex_cache_path[j].extra);
+			regex_cache_destroy(&regex_cache_subj[j]);
+			regex_cache_destroy(&regex_cache_from[j]);
+			regex_cache_destroy(&regex_cache_msgid[j]);
+			regex_cache_destroy(&regex_cache_xref[j]);
+			regex_cache_destroy(&regex_cache_path[j]);
 		}
 		free(regex_cache_subj);
 		free(regex_cache_from);
@@ -2261,45 +2257,43 @@ set_filter_scope(
 	struct t_group *group)
 {
 	int i, num, inscope;
-	struct t_filter *ptr, *prev;
+	struct t_filter *ptr;
 
 	inscope = num = group->glob_filter->num;
-	prev = ptr = group->glob_filter->filter;
+	ptr = group->glob_filter->filter;
 
 	for (i = 0; i < num; i++) {
 		ptr[i].inscope = TRUE;
-		ptr[i].next = (struct t_filter *) 0;
 		if (ptr[i].scope != NULL) {
 			if (!match_group_list(group->name, ptr[i].scope)) {
 				ptr[i].inscope = FALSE;
 				inscope--;
 			}
 		}
-		if (i != 0 && ptr[i].inscope)
-			prev = prev->next = &ptr[i];
 	}
 	return inscope;
 }
 
 
 /*
- * This will come in useful for filtering on non-overview hdr fields
+ * This will come in useful for filtering on non-overview hdr fields.
+ * See also art.c:get_path_header().
  */
 #if 0
 static FILE *
 open_xhdr_fp(
 	char *header,
-	long min,
-	long max)
+	t_artnum min,
+	t_artnum max)
 {
 #	ifdef NNTP_ABLE
 	if (read_news_via_nntp && !read_saved_news && nntp_caps.hdr_cmd) {
 		char buf[NNTP_STRLEN];
 
-		snprintf(buf, sizeof(buf), "%s %s %ld-%ld", nntp_caps.hdr_cmd, header, min, max);
+		snprintf(buf, sizeof(buf), "%s %s %"T_ARTNUM_PFMT,"-%"T_ARTNUM_PFMT,, nntp_caps.hdr_cmd, header, min, max);
 		return (nntp_command(buf, OK_HEAD, NULL, 0));
 	} else
 #	endif /* NNTP_ABLE */
-		return (FILE *) 0;		/* Some trick implementation for local spool... */
+		return (FILE *) 0;		/* Some tricky implementation for local spool... */
 }
 #endif /* 0 */
