@@ -3,7 +3,7 @@
  *  Module    : mail.c
  *  Author    : I. Lea
  *  Created   : 1992-10-02
- *  Updated   : 2022-02-19
+ *  Updated   : 2023-08-23
  *  Notes     : Mail handling routines for creating pseudo newsgroups
  *
  * Copyright (c) 1992-2023 Iain Lea <iain@bricbrac.de>
@@ -347,6 +347,11 @@ open_newsgroups_fp(
 #			endif /* DEBUG */
 								fprintf(result, "%s\n", str_trim(ptr));
 							}
+#			ifdef DEBUG
+							if ((debug & DEBUG_NNTP) && !verbose)
+								debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#			endif /* DEBUG */
+
 #		else
 							put_server(buff);
 							*buff = '\0';
@@ -373,6 +378,10 @@ open_newsgroups_fp(
 #			endif /* DEBUG */
 						fprintf(result, "%s\n", str_trim(ptr));
 					}
+#			ifdef DEBUG
+					if ((debug & DEBUG_NNTP) && !verbose)
+						debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#			endif /* DEBUG */
 				}
 				/* TODO: add 483 (RFC 3977) support */
 				if (no_more_wildmat == ERR_NOAUTH || no_more_wildmat == NEED_AUTHINFO) {
@@ -523,6 +532,10 @@ read_groups_descriptions(
 		if (++count % 100 == 0)
 			spin_cursor();
 	}
+#	if defined(DEBUG) && defined(NNTP_ABLE)
+	if ((debug & DEBUG_NNTP) && !verbose)
+		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG && NNTP_ABLE */
 	FreeIfNeeded(groupname);
 }
 
