@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2022-04-14
+ *  Updated   : 2023-05-03
  *  Notes     :
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -226,10 +226,22 @@ main(
 	 */
 	if (!nntp_server || !*nntp_server)
 		nntp_server = getserverbyfile(NNTP_SERVER_FILE);
-	if (read_news_via_nntp && !read_saved_news && nntp_open()) {
-		nntp_close(FALSE);
-		free_all_arrays();
-		giveup();
+
+	open_msglog(); /* depends on nntp_server */
+
+	if (read_news_via_nntp && !read_saved_news) {
+		if (use_nntps && tintls_init()) {
+			tintls_exit();
+			free_all_arrays();
+			giveup();
+		}
+
+		if (nntp_open()) {
+			nntp_close(FALSE);
+			tintls_exit();
+			free_all_arrays();
+			giveup();
+		}
 	}
 
 	read_server_config();
@@ -454,15 +466,16 @@ main(
 
 /*
  * process command line options
- * [01235789beEFijJkKLOtTyY] are unused
+ * [01235789beEFijJKLOtyY] are unused
  * [W] is reserved
- * [BCPU] have been in use at some time, but now are unused:
+ * [BPU] have been in use at some time, but now are unused:
  *   B BBS mode (M_AMIGA only)
- *   C count articles
  *   P purge group index files of articles that no longer exist
  *   U update index files in background
+ * reused with different function:
+ *   C was count articles, now is activate COMPRESS DEFLATE
  */
-#define OPTIONS "46aAcdD:f:g:G:hHI:lm:M:nNop:qQrRs:SuvVwxXzZ"
+#define OPTIONS "46aAcCdD:f:g:G:hHI:klm:M:nNop:qQrRs:STuvVwxXzZ"
 
 static void
 read_cmd_line_options(
@@ -541,6 +554,26 @@ read_cmd_line_options(
 				catchup = TRUE;
 				break;
 
+			case 'C':
+#ifdef NNTP_ABLE
+#	ifdef USE_ZLIB
+				use_compress = TRUE;
+#	else
+				error_message(2, _(txt_option_not_enabled), "-DUSE_ZLIB");
+				free_all_arrays();
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#	endif /* USE_ZLIB */
+#else
+				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				free_all_arrays();
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NNTP_ABLE */
+				break;
+
 			case 'd':
 				show_description = FALSE;
 				break;
@@ -571,6 +604,25 @@ read_cmd_line_options(
 			case 'g':	/* select alternative NNTP-server, implies -r */
 #ifdef NNTP_ABLE
 				my_strncpy(cmdline.nntpserver, optarg, sizeof(cmdline.nntpserver) - 1);
+				{ /* ":port" suffix - no IPv6-address support yet */
+					char *p;
+					unsigned short i;
+
+					if ((p = strchr(cmdline.nntpserver, ':')) != NULL) {
+						if (strrchr(cmdline.nntpserver, ':') == p) {
+							*p++ = '\0';
+
+							if ((i = (unsigned short) atoi(p)) != 0)
+								nntp_tcp_port = i;
+#	ifdef DEBUG
+							else {
+								if (debug & DEBUG_MISC) /* -> lang.c and _()? */
+									wait_message(3, "Port isn't numeric: %s:%s\n", cmdline.nntpserver, p);
+							}
+#	endif /* DEBUG */
+						}
+					}
+				}
 				cmdline.args |= CMDLINE_NNTPSERVER;
 				read_news_via_nntp = TRUE;
 #else
@@ -593,6 +645,19 @@ read_cmd_line_options(
 				joinpath(index_newsdir, sizeof(index_newsdir), optarg, INDEX_NEWSDIR);
 				break;
 
+			case 'k':
+#ifdef NNTPS_ABLE
+				insecure_nntps = TRUE;
+				use_nntps = TRUE;
+#else
+				error_message(2, _(txt_option_not_enabled), "--with-nntps");
+				free_all_arrays();
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NNTPS_ABLE */
+				break;
+
 			case 'l':
 				list_active = TRUE;
 				break;
@@ -689,6 +754,18 @@ read_cmd_line_options(
 				batch_mode = TRUE;
 				break;
 
+			case 'T':
+#ifdef NNTPS_ABLE
+				use_nntps = TRUE;
+#else
+				error_message(2, _(txt_option_not_enabled), "--with-nntps");
+				free_all_arrays();
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NNTPS_ABLE */
+				break;
+
 			case 'u':	/* update index files */
 				batch_mode = TRUE;
 				update_index = TRUE;
@@ -748,6 +825,18 @@ read_cmd_line_options(
 				exit(EXIT_SUCCESS);
 		}
 	}
+
+#ifdef NNTP_ABLE
+	if (nntp_tcp_port == 0) {
+#ifdef NNTPS_ABLE
+		if (use_nntps)
+			nntp_tcp_port = nntps_tcp_default_port;
+		else
+#endif /* NNTPS_ABLE */
+			nntp_tcp_port = nntp_tcp_default_port;
+	}
+#endif /* NNTP_ABLE*/
+
 	cmdargs = argv;
 	num_cmdargs = optind;
 	max_cmdargs = argc;
@@ -914,6 +1003,9 @@ usage(
 #endif /* NNTP_ABLE */
 
 	error_message(2, _(txt_usage_catchup));
+#if defined(NNTP_ABLE) && defined(USE_ZLIB)
+	error_message(2, _(txt_usage_compress));
+#endif /* NNTP_ABLE && USE_ZLIB */
 	error_message(2, _(txt_usage_dont_show_descriptions));
 
 #ifdef DEBUG
@@ -934,6 +1026,13 @@ usage(
 	error_message(2, _(txt_usage_help_message));
 	error_message(2, _(txt_usage_help_information), theProgname);
 	error_message(2, _(txt_usage_index_newsdir), index_newsdir);
+
+#ifdef NNTP_ABLE
+#	ifdef NNTPS_ABLE
+	error_message(2, _(txt_usage_use_insecure_nntps));
+#	endif /* NNTPS_ABLE */
+#endif /* NNTP_ABLE */
+
 	error_message(2, _(txt_usage_read_only_active));
 	error_message(2, _(txt_usage_maildir), tinrc.maildir);
 	error_message(2, _(txt_usage_mail_new_news_to_user));
@@ -942,7 +1041,11 @@ usage(
 	error_message(2, _(txt_usage_post_postponed_arts));
 
 #ifdef NNTP_ABLE
-	error_message(2, _(txt_usage_port), nntp_tcp_port);
+#	ifdef NNTPS_ABLE
+	error_message(2, _(txt_usage_port), use_nntps ? nntps_tcp_default_port : nntp_tcp_default_port);
+#	else
+	error_message(2, _(txt_usage_port), nntp_tcp_default_port);
+#	endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
 
 	error_message(2, _(txt_usage_dont_check_new_newsgroups));
@@ -956,6 +1059,11 @@ usage(
 	error_message(2, _(txt_usage_read_saved_news));
 	error_message(2, _(txt_usage_savedir), tinrc.savedir);
 	error_message(2, _(txt_usage_save_new_news));
+#ifdef NNTP_ABLE
+#	ifdef NNTPS_ABLE
+	error_message(2, _(txt_usage_use_nntps));
+#	endif /* NNTPS_ABLE */
+#endif /* NNTP_ABLE */
 	error_message(2, _(txt_usage_update_index_files));
 	error_message(2, _(txt_usage_verbose));
 	error_message(2, _(txt_usage_version));
@@ -1095,5 +1203,7 @@ giveup(
 		EndWin();
 		Raw(FALSE);
 	}
+	close_msglog();
+
 	exit(EXIT_FAILURE);
 }
