@@ -3,7 +3,7 @@
  *  Module    : nntplib.c
  *  Author    : S. Barber & I. Lea
  *  Created   : 1991-01-12
- *  Updated   : 2021-02-27
+ *  Updated   : 2023-08-10
  *  Notes     : NNTP client routines taken from clientlib.c 1.5.11 (1991-02-10)
  *  Copyright : (c) Copyright 1991-99 by Stan Barber & Iain Lea
  *              Permission is hereby granted to copy, reproduce, redistribute
@@ -25,6 +25,10 @@
 #	include "tnntp.h"
 #endif /* !TNNTP_H */
 
+#ifdef USE_ZLIB
+#	include <zlib.h>
+#endif /* USE_ZLIB */
+
 char *nntp_server = NULL;
 #ifdef NO_POSTING
 	t_bool can_post = FALSE;
@@ -39,16 +43,16 @@ char *nntp_server = NULL;
 	t_bool did_reconnect = FALSE;
 #endif /* NNTP_ABLE */
 
-static TCP *nntp_rd_fp = NULL;
-
 #ifdef NNTP_ABLE
-	static TCP *nntp_wr_fp = NULL;
 	/* Copy of last NNTP command sent, so we can retry it if needed */
 	static char last_put[NNTP_STRLEN];
 	static constext *xover_cmds = "XOVER";
 	static constext *xhdr_cmds = "XHDR";
 	/* Set so we don't reconnect just to QUIT */
 	static t_bool quitting = FALSE;
+#	ifdef USE_ZLIB
+	static t_bool deflate_active = FALSE;
+#	endif /* USE_ZLIB */
 #endif /* NNTP_ABLE */
 
 /*
@@ -68,30 +72,52 @@ static TCP *nntp_rd_fp = NULL;
 #	ifdef DECNET
 		static int get_dnet_socket(char *machine, char *service);
 #	endif /* DECNET */
+	static ssize_t nntp_read(int fd, void *tls, void *buf, size_t n);
+
+	struct simplebuf {
+		unsigned char buf[4096];
+		unsigned lb; /* lower bound */
+		unsigned ub; /* upper bound */
+	};
+
+	struct nntpbuf {
+		struct simplebuf rd;
+		struct simplebuf wr;
+		int fd;
+#	ifdef USE_ZLIB
+		z_streamp z_wr;
+		z_streamp z_rd;
+		unsigned char* z_wr_buf;
+		unsigned char* z_rd_buf;
+#	endif /* USE_ZLIB */
+		void *tls_ctx;
+};
+
+#	ifdef USE_ZLIB
+/* because compression can make the buffer increase, choose a larger size than
+ * for the uncompressed data */
+#		define DEFLATE_BUFSZ (5000U)
+#		define NNTPBUF_INITIALIZER { { {0}, 0, 0 }, { {0}, 0, 0 }, -1, NULL, NULL, NULL, NULL, NULL }
+#	else /* USE_ZLIB */
+#		define NNTPBUF_INITIALIZER { { {0}, 0, 0 }, { {0}, 0, 0 }, -1, NULL }
+#	endif /* USE_ZLIB */
+
+	static struct nntpbuf nntp_buf = NNTPBUF_INITIALIZER;
+
+#	ifdef USE_ZLIB
+		static void enable_deflate(struct nntpbuf* buf);
+#	endif /* USE_ZLIB */
+	static int nntpbuf_refill(struct nntpbuf *buf);
+	static int nntpbuf_flush(struct nntpbuf* buf);
+	static int nntpbuf_puts(const char* data, struct nntpbuf* buf);
+	static int nntpbuf_getc(struct nntpbuf *buf);
+	static int nntpbuf_ungetc(int c, struct nntpbuf *buf);
+	static char *nntpbuf_gets(char *s, int size, struct nntpbuf *buf);
+	static void nntpbuf_close(struct nntpbuf *buf);
+	static int nntpbuf_is_open(struct nntpbuf *buf);
 #endif /* NNTP_ABLE */
 
 
-/*
- * Return the actual fd in use for the nntp read-side socket
- * This is a leak of internal state and should go away if possible
- */
-FILE *
-get_nntp_fp(
-	FILE *fp)
-{
-	return (fp == FAKE_NNTP_FP ? nntp_rd_fp : fp);
-}
-
-
-#if 0 /* unused */
-FILE *
-get_nntp_wr_fp(
-	FILE *fp)
-{
-	return (fp == FAKE_NNTP_FP ? nntp_wr_fp : fp);
-}
-#endif /* 0 */
-
 /*
  * getserverbyfile(file)
  *
@@ -135,6 +161,28 @@ getserverbyfile(
 #ifdef NNTP_ABLE
 	if (cmdline.args & CMDLINE_NNTPSERVER) {
 		get_nntpserver(buf, sizeof(buf), cmdline.nntpserver);
+		/*
+		 * - given port in NEWSRCTABLE_FILE overrides -p, -T and $NNTPPORT
+		 * - no IPv6 address support yet (should be simple if address is
+		 *   brackets)
+		 *
+		 * news.example.com[:123]    ~/.tin/${NNTPSERVER-localhost}/.newsrc  ex
+		 */
+		if ((cp = strchr(buf, ':')) != NULL) { /* >= 1 x ':' in servername? */
+			if (strrchr(buf, ':') == cp) { /* == 1 x ':' in servername? otherwise (IPv6, syntaxerror) skip */
+				int i;
+
+				*cp++ = '\0';
+				if ((i = atoi(cp)) != 0)
+					nntp_tcp_port = (unsigned short) i;
+#	ifdef DEBUG
+				else {
+					if (debug & DEBUG_MISC) /* -> lang.c and _()? */
+						wait_message(3, "Port in %s isn't numeric: %s:%s\n", local_newsrctable_file, buf, cp);
+				}
+#	endif /* DEBUG */
+			}
+		}
 #	ifdef HAVE_SETENV
 		setenv("NNTPSERVER", buf, 1);
 #	else
@@ -151,6 +199,21 @@ getserverbyfile(
 
 	if ((cp = getenv("NNTPSERVER")) != NULL) {
 		get_nntpserver(buf, sizeof(buf), cp);
+		if ((cp = strchr(buf, ':')) != NULL) {
+			if (strrchr(buf, ':') == cp) {
+				int i;
+
+				*cp++ = '\0';
+				if ((i = atoi(cp)) != 0)
+					nntp_tcp_port = (unsigned short) i;
+#	ifdef DEBUG
+				else {
+					if (debug & DEBUG_MISC) /* -> lang.c and _()? */
+						wait_message(3, "Port in $NNTPSERVER isn't numeric: %s:%s\n", buf, cp);
+				}
+#	endif /* DEBUG */
+			}
+		}
 		return buf;
 	}
 
@@ -210,7 +273,7 @@ server_init(
 	char temp[256];
 	char *service = strncpy(temp, cservice, sizeof(temp) - 1); /* ...calls non-const funcs */
 #	endif /* !INET6 */
-	int sockt_rd, sockt_wr;
+	int sock_fd;
 
 #	ifdef DECNET
 	char *cp;
@@ -219,57 +282,44 @@ server_init(
 
 	if (cp && cp[1] == ':') {
 		*cp = '\0';
-		sockt_rd = get_dnet_socket(machine, service);
+		sock_fd = get_dnet_socket(machine, service);
 	} else
-		sockt_rd = get_tcp_socket(machine, service, port);
+		sock_fd = get_tcp_socket(machine, service, port);
 #	else
 #		ifdef INET6
-	sockt_rd = get_tcp6_socket(machine, port);
+	sock_fd = get_tcp6_socket(machine, port);
 #		else
-	sockt_rd = get_tcp_socket(machine, service, port);
+	sock_fd = get_tcp_socket(machine, service, port);
 #		endif /* INET6 */
 #	endif /* DECNET */
 
-	if (sockt_rd < 0)
-		return sockt_rd;
-
-	/*
-	 * Now we'll make file pointers (i.e., buffered I/O) out of
-	 * the socket file descriptor. Note that we can't just
-	 * open a fp for reading and writing -- we have to open
-	 * up two separate fp's, one for reading, one for writing.
-	 */
-
-	if ((nntp_rd_fp = (TCP *) s_fdopen(sockt_rd, "r")) == NULL) {
-		perror("server_init: fdopen() #1");
-		s_close(sockt_rd);
-		return -errno;
-	}
-
-	if ((sockt_wr = s_dup(sockt_rd)) < 0) {
-		perror("server_init: dup()");
-		s_fclose(nntp_rd_fp);
-		nntp_rd_fp = NULL;
-		return -errno;
-	}
+	if (sock_fd < 0)
+		return sock_fd;
 
 #	ifdef TLI /* Transport Level Interface */
-	if (t_sync(sockt_rd) < 0) {	/* Sync up new fd with TLI */
+	if (t_sync(sock_fd) < 0) {	/* Sync up new fd with TLI */
 		t_error("server_init: t_sync()");
-		s_fclose(nntp_rd_fp);
-		nntp_rd_fp = NULL;
+		close(sock_fd);
 		return -EPROTO;
 	}
-#	else
-	if ((nntp_wr_fp = (TCP *) s_fdopen(sockt_wr, "w")) == NULL) {
-		perror("server_init: fdopen() #2");
-		s_close(sockt_wr);
-		s_fclose(nntp_rd_fp);
-		nntp_rd_fp = NULL;
-		return -errno;
-	}
 #	endif /* TLI */
 
+#	ifdef NNTPS_ABLE
+	if (use_nntps) {
+		int result;
+
+		result = tintls_open(machine, sock_fd, &nntp_buf.tls_ctx);
+		if (result < 0)
+			return result;
+
+		result = tintls_handshake(nntp_buf.tls_ctx);
+		if (result < 0)
+			return result;
+	}
+#	endif /* NNTPS_ABLE */
+
+	nntp_buf.fd = sock_fd;
+
 	last_put[0] = '\0';		/* no retries in get_respcode */
 	/*
 	 * Now get the server's signon message
@@ -523,7 +573,7 @@ get_tcp_socket(
 		my_fprintf(stderr, _(txt_connection_to), (char *) inet_ntoa(sock_in.sin_addr));
 		perror("");
 #			endif /* HAVE_INET_NTOA */
-		(void) s_close(s);
+		(void) close(s);
 	}
 
 	if (x < 0) {
@@ -552,7 +602,7 @@ get_tcp_socket(
 	if (connect(s, (struct sockaddr *) &sock_in) < 0) {
 		save_errno = errno;
 		perror("connect");
-		(void) s_close(s);
+		(void) close(s);
 		return -save_errno;
 	}
 
@@ -571,7 +621,7 @@ get_tcp_socket(
 	if (connect(s, (struct sockaddr *) &sock_in, sizeof(sock_in)) < 0) {
 		save_errno = errno;
 		perror("connect");
-		(void) s_close(s);
+		(void) close(s);
 		return -save_errno;
 	}
 
@@ -620,6 +670,9 @@ get_tcp6_socket(
 #			define ADDRFAM	AF_INET
 #		endif /* PF_UNSPEC */
 #	endif /* AF_UNSPEC */
+#	ifndef AF_INET6 /* i.e. sco3.2v5.0.7 */
+#		define  AF_INET6 AF_INET
+#	endif /* ! AF_INET6 */
 	memset(&hints, 0, sizeof(hints));
 /*	hints.ai_flags = AI_CANONNAME; */
 	hints.ai_family = (force_ipv4 ? AF_INET : (force_ipv6 ? AF_INET6 : ADDRFAM));
@@ -637,7 +690,7 @@ get_tcp6_socket(
 		}
 		if (connect(s, res->ai_addr, res->ai_addrlen) != 0) {
 			ec = errno;
-			s_close(s);
+			close(s);
 		} else {
 			es = ec = err = 0;
 			break;
@@ -723,7 +776,7 @@ get_dnet_socket(
 
 	if (connect(s, (struct sockaddr *) &sdn, sizeof(sdn)) < 0) {
 		nerror("connect");
-		s_close(s);
+		close(s);
 		return -1;
 	}
 
@@ -748,7 +801,7 @@ void
 u_put_server(
 	const char *string)
 {
-	s_puts(string, nntp_wr_fp);
+	nntpbuf_puts(string, &nntp_buf);
 #	ifdef DEBUG
 	if (debug & DEBUG_NNTP) {
 		if (strcmp(string, "\r\n"))
@@ -778,15 +831,34 @@ put_server(
 {
 	if (*string && strlen(string)) {
 		DEBUG_IO((stderr, "put_server(%s)\n", string));
-		s_puts(string, nntp_wr_fp);
-		s_puts("\r\n", nntp_wr_fp);
+		nntpbuf_puts(string, &nntp_buf);
+		nntpbuf_puts("\r\n", &nntp_buf);
+
 #	ifdef DEBUG
-		if (debug & DEBUG_NNTP)
-			debug_print_file("NNTP", ">>>%s%s", logtime(), string);
+		if (debug & DEBUG_NNTP) {
+			if (verbose)	/* only log password when running verbose */
+				debug_print_file("NNTP", ">>>%s%s", logtime(), string);
+			else {
+				char *c = my_strdup(string);
+				int l = 0;
+
+				if (!strncmp(string,"AUTHINFO PASS", 13))
+					l = 13;
+				if (!l && !strncmp(string,"AUTHINFO SASL PLAIN", 19))
+					l = 19;
+
+				if (l)
+					*(c+l) = '\0';
+
+				debug_print_file("NNTP", ">>>%s%s%s", logtime(), c, l ? " [data hidden, rerun with -v]" : "");
+				free(c);
+			}
+		}
 #	endif /* DEBUG */
+
 		/*
-		 * remember the last command we wrote to be able to resend it after a
-		 * reconnect. reconnection is handled by get_server()
+		 * remember the last command we wrote to be able to resend it after
+		 * a reconnect. reconnection is handled by get_server()
 		 *
 		 * don't cache "LIST [ACTIVE|COUNTS|NEWSGROUPS] something" as we
 		 * would need to resend all of them but we remember just the last
@@ -802,7 +874,7 @@ put_server(
 		else if (!strncmp(string, "LIST NEWSGROUPS ", 16))
 			last_put[15] = '\0'; /* "LIST NEWSGROUPS" */
 	}
-	(void) s_flush(nntp_wr_fp);
+	(void) nntpbuf_flush(&nntp_buf);
 }
 
 
@@ -821,11 +893,7 @@ reconnect(
 	 * Tear down current connection
 	 * Close the NNTP connection with prejudice
 	 */
-	if (nntp_wr_fp)
-		s_fclose(nntp_wr_fp);
-	if (nntp_rd_fp)
-		s_fclose(nntp_rd_fp);
-	nntp_rd_fp = nntp_wr_fp = NULL;
+	nntpbuf_close(&nntp_buf);
 
 	if (!tinrc.auto_reconnect)
 		ring_bell();
@@ -845,6 +913,7 @@ reconnect(
 	 */
 	if (retry > NNTP_TRY_RECONNECT || (!tinrc.auto_reconnect && prompt_yn(_(txt_reconnect_to_news_server), TRUE) != 1)) {
 		if (!strcmp("POST", last_put)) {
+			/* TODO: also/only postpone_article(article_name) ? */
 			unlink(backup_article_name(article_name));
 			rename_file(article_name, dead_article);
 			if (tinrc.keep_dead_articles)
@@ -862,6 +931,9 @@ reconnect(
 
 	/* reset signal_context */
 	signal_context = save_signal_context;
+#	if defined(HAVE_ALARM) && defined(SIGALRM)
+		alarm((unsigned) TIN_NNTP_TIMEOUT);
+#	endif /* HAVE_ALARM && SIGALRM */
 
 	clear_message();
 	strcpy(buf, last_put);			/* Keep copy here, it will be clobbered a lot otherwise */
@@ -872,7 +944,7 @@ reconnect(
 			DEBUG_IO((stderr, _("Rejoin current group\n")));
 			snprintf(last_put, sizeof(last_put), "GROUP %s", curr_group->name);
 			put_server(last_put);
-			if (s_gets(last_put, NNTP_STRLEN, nntp_rd_fp) == NULL)
+			if (nntpbuf_gets(last_put, NNTP_STRLEN, &nntp_buf) == NULL)
 				*last_put = '\0';
 #	ifdef DEBUG
 			if (debug & DEBUG_NNTP)
@@ -886,10 +958,48 @@ reconnect(
 		retry = NNTP_TRY_RECONNECT;
 	}
 
+#	if defined(HAVE_ALARM) && defined(SIGALRM)
+		alarm(0);
+#	endif /* HAVE_ALARM && SIGALRM */
+
 	return retry;
 }
 
 
+int
+fgetc_server(
+	FILE *stream)
+{
+	int c = EOF;
+
+	if (stream != FAKE_NNTP_FP) {
+		DEBUG_IO((stderr, "fgetc_server: BAD fp\n"));
+		return EOF;
+	}
+
+	c = nntpbuf_getc(&nntp_buf);
+	DEBUG_IO((stderr, "fgetc_server: %c / %d\n", c, c));
+
+	return c;
+}
+
+
+int
+ungetc_server(
+	int c,
+	FILE *stream)
+{
+	if (stream != FAKE_NNTP_FP) {
+		DEBUG_IO((stderr, "fgetc_server: BAD fp\n"));
+		return EOF;
+	}
+
+	DEBUG_IO((stderr, "ungetc_server: %c / %d\n", c, c));
+
+	return nntpbuf_ungetc(c, &nntp_buf);
+}
+
+
 /*
  * Read a line of data from the NNTP socket. If something gives, do reconnect
  *
@@ -922,13 +1032,13 @@ get_server(
 	 *   -the network connection went down
 	 */
 #	if defined(HAVE_ALARM) && defined(SIGALRM)
-	alarm((unsigned) tinrc.nntp_read_timeout_secs);
+	alarm((unsigned) TIN_NNTP_TIMEOUT);
 #	endif /* HAVE_ALARM && SIGALRM */
-	while (nntp_rd_fp == NULL || s_gets(string, size, nntp_rd_fp) == NULL) {
+	while (!nntpbuf_is_open(&nntp_buf) || nntpbuf_gets(string, size, &nntp_buf) == NULL) {
 		if (errno == EINTR) {
 			errno = 0;
 #	if defined(HAVE_ALARM) && defined(SIGALRM)
-			alarm((unsigned) tinrc.nntp_read_timeout_secs);		/* Restart the timer */
+			alarm((unsigned) TIN_NNTP_TIMEOUT);		/* Restart the timer */
 #	endif /* HAVE_ALARM && SIGALRM */
 			continue;
 		}
@@ -951,7 +1061,7 @@ get_server(
 		 */
 		if (strcmp(last_put, "QUIT")) {
 			/*
-			 * Typhoon v2.1.1.363 colses the connection right after an unknown
+			 * Typhoon v2.1.1.363 closes the connection right after an unknown
 			 * command, (i.e. CAPABILITIES) so we avoid to reissue it on a
 			 * reconnect if it was the last command.
 			 */
@@ -994,8 +1104,7 @@ static void
 close_server(
 	t_bool send_no_quit)
 {
-	if (!send_no_quit && nntp_wr_fp && nntp_rd_fp) {
-
+	if (!send_no_quit && nntpbuf_is_open(&nntp_buf)) {
 		if (!batch_mode || verbose) {
 			char *msg;
 
@@ -1007,12 +1116,7 @@ close_server(
 		nntp_command("QUIT", OK_GOODBYE, NULL, 0);
 		quitting = TRUE;										/* Don't reconnect just for this */
 	}
-	if (nntp_wr_fp)
-		(void) s_fclose(nntp_wr_fp);
-	if (nntp_rd_fp)
-		(void) s_fclose(nntp_rd_fp);
-	s_end();
-	nntp_wr_fp = nntp_rd_fp = NULL;
+	nntpbuf_close(&nntp_buf);
 }
 
 
@@ -1073,6 +1177,7 @@ check_extensions(
 			nntp_caps.sasl = SASL_NONE;
 			nntp_caps.compress = FALSE;
 			nntp_caps.compress_algorithm = COMPRESS_NONE;
+			/* nntp_caps.maxartnum will be init it nntp_open() */
 #	if 0
 			nntp_caps.streaming = FALSE;
 			nntp_caps.ihave = FALSE;
@@ -1240,12 +1345,31 @@ check_extensions(
 							}
 						}
 					}
-#		if 0 /* we don't need these */
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+					/*
+					 * MAXARTNUM - <tnqm14$35bas$1@news.trigofacile.com>
+					 *
+					 * if server responds with MAXARTNUM we (re)parse it
+					 * as we may have changed the state (auth) and it's
+					 * the servers job to not advertised MAXARTNUM again
+					 * after it had been used ...
+					 */
+					else if (!strncasecmp(ptr, "MAXARTNUM", 9) && nntp_caps.maxartnum == T_ARTNUM_CONST(0)) {
+						d = ptr + 9;
+						d = strpbrk(d, " \t");
+						while (d != NULL && (d + 1 < (ptr + strlen(ptr)))) {
+							d++;
+							nntp_caps.maxartnum = MIN(atoartnum(d), ARTNUM_MAX);
+							d = strpbrk(d, " \t");
+						}
+					}
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+#	if 0 /* we don't need these */
 					else if (!strcasecmp(ptr, "IHAVE"))
 						nntp_caps.ihave = TRUE;
 					else if (!strcasecmp(ptr, "STREAMING"))
 						nntp_caps.streaming = TRUE;
-#		endif /* 0 */
+#	endif /* 0 */
 				/* XZVER, XZHDR, ... */
 				} else
 					nntp_caps.type = NONE;
@@ -1258,17 +1382,22 @@ check_extensions(
 		 * you use tin on a XanaNewz 2 Server comment out the following
 		 * case.
 		 */
-#		if 1
+#	if 1
 		case ERR_GOODBYE:
 			ret = i;
 			error_message(2, "%s", buf);
 			break;
-#		endif /* 1 */
+#	endif /* 1 */
 
 		default:
 			break;
 	}
 
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+	if (nntp_caps.maxartnum <= T_ARTNUM_CONST(2147483647)) /* RFC 3977 "Article numbers MUST lie between 1 and 2,147,483,647, inclusive." */
+		nntp_caps.maxartnum = T_ARTNUM_CONST(0);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+
 	return ret;
 }
 
@@ -1352,7 +1481,7 @@ nntp_open(
 {
 #ifdef NNTP_ABLE
 	char *linep;
-	char line[NNTP_STRLEN]= { '\0' };
+	char line[NNTP_STRLEN] = { '\0' };
 	int ret;
 	t_bool sec = FALSE;
 	/* It appears that is_reconnect guards code that should be run only once */
@@ -1372,22 +1501,21 @@ nntp_open(
 		return -EHOSTUNREACH;
 	}
 
-	if (!batch_mode || verbose) {
-		if (nntp_tcp_port != IPPORT_NNTP)
-			wait_message(0, _(txt_connecting_port), nntp_server, nntp_tcp_port);
-		else
-			wait_message(0, _(txt_connecting), nntp_server);
-	}
+	if (!batch_mode || verbose)
+		wait_message(0, _(txt_connecting_port), nntp_server, nntp_tcp_port);
+
+	if ((!batch_mode || verbose) && use_nntps)
+		my_fputc('\n', stdout);
 
 #	ifdef DEBUG
 	if ((debug & DEBUG_NNTP) && verbose > 1)
-		debug_print_file("NNTP", "nntp_open() %s:%d", nntp_server, nntp_tcp_port);
+		debug_print_file("NNTP", "nntp_open() %s:%u", nntp_server, nntp_tcp_port);
 #	endif /* DEBUG */
 
 	ret = server_init(nntp_server, NNTP_TCP_NAME, nntp_tcp_port, line, sizeof(line));
 	DEBUG_IO((stderr, "server_init returns %d,%s\n", ret, line));
 
-	if ((!batch_mode || verbose) && ret >= 0)
+	if ((!batch_mode || verbose) && ret >= 0 && !use_nntps)
 		my_fputc('\n', stdout);
 
 #	ifdef DEBUG
@@ -1438,6 +1566,10 @@ nntp_open(
 		STRCPY(bug_nntpserver1, linep);
 	}
 
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+	nntp_caps.maxartnum = T_ARTNUM_CONST(0);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+
 	/*
 	 * Find out which NNTP extensions are available
 	 * - Typhoon v2.1.1.363 closes the connection after an unknown command
@@ -1538,7 +1670,7 @@ nntp_open(
 
 		/*
 		 * Show user last server response line, do some nice formatting if
-		 * response is longer than a screen wide.
+		 * response is longer than the screen width.
 		 *
 		 * TODO: This only breaks the line once, but the response could be
 		 * longer than two lines ...
@@ -1568,6 +1700,8 @@ nntp_open(
 	/*
 	 * If CAPABILITIES failed, check if NNTP supports XOVER or OVER command
 	 * We have to check that we _don't_ get an ERR_COMMAND
+	 *
+	 * TODO: this should be done when the command is first used!
 	 */
 	if (nntp_caps.type != CAPABILITIES) {
 		int i, j = 0;
@@ -1698,6 +1832,10 @@ nntp_open(
 	}
 #	endif /* 0 */
 
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+	set_maxartnum(is_reconnect);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+
 	if (!is_reconnect && !batch_mode && show_description && check_for_new_newsgroups) {
 		/*
 		 * TODO:
@@ -1710,6 +1848,16 @@ nntp_open(
 
 	is_reconnect = TRUE;
 
+#	ifdef USE_ZLIB
+	/*
+	 * Enable compression if available
+	 * Note: after enabling compression, authentication shall not work anymore
+	 */
+	if (nntp_caps.compress && use_compress) {
+		if ((nntp_caps.compress_algorithm & COMPRESS_DEFLATE) == COMPRESS_DEFLATE)
+			enable_deflate(&nntp_buf);
+	}
+#	endif /* USE_ZLIB */
 #endif /* NNTP_ABLE */
 
 	DEBUG_IO((stderr, "nntp_open okay\n"));
@@ -1781,6 +1929,10 @@ get_only_respcode(
 	 * what about other LIST cmds? (ACTIVE|COUNTS|OVERVIEW.FMT|...)
 	 */
 	if (last_put[0] != '\0' && ((respcode == ERR_FAULT && (!strncmp(last_put, "ARTICLE", 7) || !strcmp(last_put, "POST") || !strcmp(last_put, "LIST"))) || respcode == ERR_GOODBYE || respcode == OK_GOODBYE) && strcmp(last_put, "QUIT")) {
+		if (respcode == ERR_GOODBYE && !strncmp(last_put, "HEAD ", 5)) {
+			/* usenetfarm may send ERR_GOODBYE in response to HEAD, we don't want to retry that */
+			return respcode;
+		}
 		/*
 		 * Maybe server timed out.
 		 * If so, retrying will force a reconnect.
@@ -1842,6 +1994,11 @@ get_respcode(
 
 	respcode = get_only_respcode(message, mlen);
 	if ((respcode == ERR_NOAUTH) || (respcode == NEED_AUTHINFO)) {
+#	ifdef USE_ZLIB
+		if (deflate_active) /* Do not auth if compression is active */
+			tin_done(EXIT_FAILURE, _(txt_error_compression_auth), tin_progname);
+#	endif /* USE_ZLIB */
+
 		/*
 		 * Server requires authentication.
 		 */
@@ -1865,7 +2022,7 @@ get_respcode(
 			DEBUG_IO((stderr, _("Rejoin current group\n")));
 			snprintf(last_put, sizeof(last_put), "GROUP %s", curr_group->name);
 			put_server(last_put);
-			if (s_gets(last_put, NNTP_STRLEN, nntp_rd_fp) == NULL)
+			if (nntpbuf_gets(last_put, NNTP_STRLEN, &nntp_buf) == NULL)
 				*last_put = '\0';
 #	ifdef DEBUG
 			if (debug & DEBUG_NNTP)
@@ -2034,4 +2191,549 @@ list_motd(
 			break;
 	}
 }
+
+
+static ssize_t
+nntp_write(
+	int fd,
+	void *tls,
+	const void *buf,
+	size_t n)
+{
+	ssize_t bytes_written;
+
+#	ifdef NNTPS_ABLE
+	if (tls)
+		bytes_written = tintls_write(tls, buf, n);
+	else
+#	endif /* NNTPS_ABLE */
+		bytes_written = write(fd, buf, n);
+
+	return bytes_written;
+}
+
+
+ssize_t nntp_read(
+	int fd,
+	void *tls,
+	void *buf,
+	size_t n)
+{
+	ssize_t bytes_read;
+
+#	ifdef NNTPS_ABLE
+	if (tls)
+		bytes_read = tintls_read(tls, buf, n);
+	else
+#	endif /* NNTPS_ABLE */
+		bytes_read = read(fd, buf, n);
+
+	return bytes_read;
+}
+
+
+#define SZ(a) sizeof((a))
+
+#	ifdef USE_ZLIB
+static void *
+deflate_alloc(
+	void *user,
+	uInt items,
+	uInt size)
+{
+	(void) user;
+	return my_calloc(items, size);
+}
+
+
+static void
+deflate_free(
+	void *user,
+	void *ptr)
+{
+	(void) user;
+	FreeIfNeeded(ptr);
+}
+
+
+static z_streamp
+z_stream_init(
+	t_bool is_deflate)
+{
+	int result;
+
+	z_streamp strm = my_calloc(1, sizeof(z_stream));
+	strm->zalloc = deflate_alloc;
+	strm->zfree = deflate_free;
+
+	if (is_deflate)
+		result = deflateInit2(strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY);
+	else
+		result = inflateInit2(strm, -15);
+
+	if (result != Z_OK)
+		FreeAndNull(strm);
+
+	return strm;
+}
+
+
+static void
+enable_deflate(
+	struct nntpbuf* nntpbuf)
+{
+	char buf[NNTP_STRLEN];
+	int result;
+
+	if (nntpbuf->z_rd || nntpbuf->z_wr)
+		return;
+
+	nntpbuf->z_rd = z_stream_init(FALSE);
+	nntpbuf->z_wr = z_stream_init(TRUE);
+
+	if (nntpbuf->z_rd == NULL || nntpbuf->z_wr == NULL)
+		goto error_out;
+
+	nntpbuf->z_rd_buf = my_malloc(DEFLATE_BUFSZ);
+	nntpbuf->z_rd->next_in = nntpbuf->z_rd_buf;
+	nntpbuf->z_rd->avail_in = 0;
+
+	nntpbuf->z_wr_buf = my_malloc(DEFLATE_BUFSZ);
+	nntpbuf->z_wr->next_out = nntpbuf->z_wr_buf;
+	nntpbuf->z_wr->avail_out = DEFLATE_BUFSZ;
+
+	buf[0] = '\0';
+	result = new_nntp_command("COMPRESS DEFLATE", OK_COMPRESS, buf, sizeof(buf));
+
+	switch (result) {
+		case OK_COMPRESS:
+			deflate_active = TRUE;
+			return;
+
+		case ERR_COMPRESS_ALG:
+		case ERR_COMPRESS:
+		default: /* unexpected */
+			break;
+	}
+
+error_out:
+	FreeAndNull(nntpbuf->z_rd);
+	FreeAndNull(nntpbuf->z_wr);
+	FreeAndNull(nntpbuf->z_rd_buf);
+	FreeAndNull(nntpbuf->z_wr_buf);
+}
+
+
+static ssize_t
+nntpbuf_deflate_write(
+	struct nntpbuf* buf)
+{
+	int result;
+	ssize_t bytes_written = 0, bwritten;
+	t_bool deflate_again = TRUE;
+
+	buf->z_wr->next_in = buf->wr.buf + buf->wr.lb;
+	buf->z_wr->avail_in = buf->wr.ub - buf->wr.lb;
+
+	while (deflate_again) {
+		Bytef *out = buf->z_wr->next_out;
+
+		result = deflate(buf->z_wr, Z_PARTIAL_FLUSH);
+		if (result < 0)
+			return EOF;
+
+		if (buf->z_wr->avail_in > 0 || buf->z_wr->avail_out == 0)
+			deflate_again = TRUE;
+		else
+			deflate_again = FALSE;
+
+		while (buf->z_wr->avail_out < DEFLATE_BUFSZ) {
+			bwritten = nntp_write(buf->fd, buf->tls_ctx, out, DEFLATE_BUFSZ - buf->z_wr->avail_out);
+			if (bwritten < 0)
+				return EOF;
+
+			buf->z_wr->avail_out += bwritten;
+			out += bwritten;
+			bytes_written += bwritten;
+		}
+	}
+	buf->wr.lb = buf->wr.ub;
+
+	return bytes_written;
+}
+
+
+static ssize_t
+nntpbuf_inflate(
+	struct nntpbuf* buf)
+{
+	int result;
+
+	result = inflate(buf->z_rd, Z_NO_FLUSH);
+	if (result < 0 && result != Z_BUF_ERROR)
+		return EOF;
+
+	/* move leftover input data to beginning of input buffer */
+	my_memmove(buf->z_rd_buf, buf->z_rd->next_in, buf->z_rd->avail_in);
+	buf->z_rd->next_in = buf->z_rd_buf;
+
+	return (SZ(buf->rd.buf) - buf->rd.ub) - buf->z_rd->avail_out;
+}
+
+
+static ssize_t
+nntpbuf_inflate_read(
+	struct nntpbuf* buf)
+{
+	ssize_t bytes_read;
+	ssize_t bread;
+
+	buf->z_rd->next_out = buf->rd.buf + buf->rd.ub;
+	buf->z_rd->avail_out = SZ(buf->rd.buf) - buf->rd.ub;
+
+	/* call inflate unconditionally to make sure there is no pending output
+	   left, before calling the possibly blocking read below */
+	bytes_read = nntpbuf_inflate(buf);
+	if (bytes_read < 0)
+		return bytes_read;
+
+	while (bytes_read == 0) {
+		if (buf->z_rd->avail_in < DEFLATE_BUFSZ) {
+			bread = nntp_read(buf->fd, buf->tls_ctx, buf->z_rd->next_in, DEFLATE_BUFSZ - buf->z_rd->avail_in);
+			if (bread <= 0)
+				return EOF;
+
+			buf->z_rd->avail_in += bread;
+		}
+
+		bread = nntpbuf_inflate(buf);
+		if (bread < 0)
+			return bread;
+
+		bytes_read += bread;
+	}
+	return bytes_read;
+}
+#	endif /* USE_ZLIB */
+
+
+/*
+ * write data from write buffer to NNTP connection when requested
+ */
+static int
+nntpbuf_flush(
+	struct nntpbuf* buf)
+{
+	if (!buf)
+		return EOF;
+
+	while (buf->wr.ub > buf->wr.lb) {
+		ssize_t bytes_written;
+
+#	ifdef USE_ZLIB
+		if (deflate_active)
+			bytes_written = nntpbuf_deflate_write(buf);
+		else
+#	endif /* USE_ZLIB */
+			bytes_written = nntp_write(buf->fd, buf->tls_ctx, buf->wr.buf + buf->wr.lb, buf->wr.ub - buf->wr.lb);
+
+		if (bytes_written < 0)
+			return EOF;
+
+		buf->wr.lb += bytes_written;
+	}
+
+	buf->wr.ub = buf->wr.lb = 0;
+
+	return 0;
+}
+
+
+/*
+ * fputs(3) replacement using the NNTP connection
+ */
+static int
+nntpbuf_puts(
+	const char* data,
+	struct nntpbuf* buf)
+{
+	int bytes_written = 0, retval;
+	unsigned len, l;
+
+	if (!buf || SZ(buf->wr.buf) == 0 || buf->wr.lb > buf->wr.ub)
+		return EOF;
+
+	if (!data)
+		return 0;
+
+	len = strlen(data);
+	while (len) {
+		if (buf->wr.ub == SZ(buf->wr.buf)) {
+			retval = nntpbuf_flush(buf);
+
+			if (retval != 0)
+				return retval;
+		}
+
+		l = len;
+		if (l > SZ(buf->wr.buf) - buf->wr.ub)
+			l = SZ(buf->wr.buf) - buf->wr.ub;
+
+		memcpy(buf->wr.buf + buf->wr.ub, data, l);
+		buf->wr.ub += l;
+		data += l;
+		bytes_written += l;
+		len -= l;
+	}
+
+	return bytes_written;
+}
+
+
+/*
+ * internal helper to refill the read buffer using the NNTP connection
+ */
+static int
+nntpbuf_refill(
+	struct nntpbuf *buf)
+{
+	unsigned free_b;
+
+	if (buf->rd.ub == buf->rd.lb)
+		buf->rd.ub = buf->rd.lb = 0;
+
+	free_b = SZ(buf->rd.buf) - buf->rd.ub;
+	if (free_b) {
+		ssize_t bytes_read;
+
+#	ifdef USE_ZLIB
+		if (deflate_active)
+			bytes_read = nntpbuf_inflate_read(buf);
+		else
+#	endif /* USE_ZLIB */
+			bytes_read = nntp_read(buf->fd, buf->tls_ctx, buf->rd.buf + buf->rd.ub, free_b);
+
+		if (bytes_read > 0)
+			buf->rd.ub += bytes_read;
+
+		return (int) bytes_read;
+	}
+	return 0;
+}
+
+
+/*
+ * fgetc(3) replacement using the NNTP connection
+ */
+static int
+nntpbuf_getc(
+	struct nntpbuf *buf)
+{
+	int c = EOF, retval;
+
+	if (buf->rd.ub - buf->rd.lb == 0) {
+		retval = nntpbuf_refill(buf);
+
+		if (retval <= 0)
+			return retval;
+	}
+
+	c = buf->rd.buf[buf->rd.lb];
+	buf->rd.lb++;
+
+	return c;
+}
+
+
+/*
+ * ungetc(3) replacement using the NNTP connection
+ */
+static int
+nntpbuf_ungetc(
+	int c,
+	struct nntpbuf *buf)
+{
+	if (buf->rd.lb == 0) {
+		if (buf->rd.ub == SZ(buf->rd.buf)) {
+			errno = ENOSPC;
+			return EOF;
+		}
+		my_memmove(buf->rd.buf + 1, buf->rd.buf, buf->rd.ub);
+		buf->rd.lb++;
+	}
+
+	buf->rd.lb--;
+	buf->rd.buf[buf->rd.lb] = (unsigned char)c;
+
+	return c;
+}
+
+
+/*
+ * fgets(3) replacement using the NNTP connection
+ */
+static char *
+nntpbuf_gets(
+	char *s,
+	int size,
+	struct nntpbuf *buf)
+{
+	int write_at = 0, retval;
+
+	if (s == NULL || size == 0)
+		return s;
+
+	s[--size] = '\0';
+
+	while (size) {
+		if (buf->rd.ub - buf->rd.lb == 0) {
+			retval = nntpbuf_refill(buf);
+
+			if (retval <= 0)
+				return NULL;
+		}
+
+		while (size && (buf->rd.ub - buf->rd.lb) > 0) {
+			s[write_at++] = buf->rd.buf[buf->rd.lb++];
+			size--;
+
+			if (s[write_at - 1] == '\n' && size) {
+				s[write_at] = '\0';
+				return s;
+			}
+		}
+	}
+
+	return s;
+}
+
+
+static void
+nntpbuf_close(
+	struct nntpbuf *buf)
+{
+	if (!buf)
+		return;
+
+#	ifdef NNTPS_ABLE
+	if (buf->tls_ctx) {
+		int result = tintls_close(buf->tls_ctx);
+
+		if (result != 0) {
+			/* warn? */
+		}
+	}
+	buf->tls_ctx = NULL;
+#	endif /* NNTPS_ABLE */
+
+	if (buf->fd >= 0)
+		close(buf->fd);
+
+	buf->fd = -1;
+
+	buf->rd.lb = buf->rd.ub = 0;
+	buf->wr.lb = buf->wr.ub = 0;
+
+#	ifdef USE_ZLIB
+	if (deflate_active) {
+		if (buf->z_rd)
+			inflateEnd(buf->z_rd);
+		FreeAndNull(buf->z_rd);
+		FreeAndNull(buf->z_rd_buf);
+
+		if (buf->z_wr)
+			deflateEnd(buf->z_wr);
+		FreeAndNull(buf->z_wr);
+		FreeAndNull(buf->z_wr_buf);
+
+		deflate_active = FALSE;
+	}
+#	endif /* USE_ZLIB */
+}
+
+
+static int
+nntpbuf_is_open(
+	struct nntpbuf *buf)
+{
+	return buf->fd != -1;
+}
+
+#undef SZ
+
+int
+nntp_conninfo(
+	FILE *stream)
+{
+	int retval = 0;
+
+	fprintf(stream, "\nConnection details:\n");
+	fprintf(stream, "-------------------\n");
+	fprintf(stream, "NNTPSERVER    : %s\n", nntp_server);
+	fprintf(stream, "NNTPPORT      : %u\n", nntp_tcp_port);
+	if (nntp_caps.type == CAPABILITIES) {
+		if (*nntp_caps.implementation)
+			fprintf(stream, "IMPLEMENTATION: %s\n", nntp_caps.implementation);
+		if (nntp_caps.compress) {
+			fprintf(stream, "COMPRESS      :");
+			if ((nntp_caps.compress_algorithm & COMPRESS_DEFLATE) == COMPRESS_DEFLATE)
+#	ifdef USE_ZLIB
+				fprintf(stream, " DEFLATE %s\n", deflate_active ? "(enabled)" : "(inactive)");
+#	else
+				fprintf(stream, " DEFLATE (not supported)\n");
+#	endif /* USE_ZLIB */
+		}
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+		if (nntp_caps.maxartnum)
+			fprintf(stream, "MAXARTNUM     : %"T_ARTNUM_PFMT"\n", nntp_caps.maxartnum);
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
+	}
+
+#	if defined(HAVE_ALARM) && defined(SIGALRM)
+	fprintf(stream, "NNTP TIMEOUT  : %d seconds %s\n", TIN_NNTP_TIMEOUT, TIN_NNTP_TIMEOUT ? "" : "(disabled)");
+#	endif /* HAVE_ALARM && SIGALRM */
+
+#	ifdef NNTPS_ABLE
+	if (nntp_buf.tls_ctx)
+		retval = tintls_conninfo(nntp_buf.tls_ctx, stream);
+
+#	endif /* NNTPS_ABLE */
+
+	return retval;
+}
+
+
+#	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
+/*
+ * MAXARTNUM - <tnqm14$35bas$1@news.trigofacile.com>
+ */
+void
+set_maxartnum(
+	t_bool reconnect)
+{
+	char cmd[NNTP_STRLEN];
+	char line[NNTP_STRLEN] = { '\0' };
+	static int cnt = 0;
+
+	if (nntp_caps.maxartnum < T_ARTNUM_CONST(2147483647) || (!reconnect && cnt))
+		return;
+	else
+		cnt++;
+
+	snprintf(cmd, sizeof(cmd), "MAXARTNUM %"T_ARTNUM_PFMT, nntp_caps.maxartnum);
+
+	switch (new_nntp_command(cmd, OK_EXTENSIONS, line, sizeof(line))) {
+		case OK_EXTENSIONS:
+			nntp_caps.mode_reader = FALSE;
+			nntp_caps.reader = TRUE;
+			nntp_caps.list_newsgroups = TRUE;
+			nntp_caps.list_active = TRUE;
+			break;
+
+		default:
+			nntp_caps.maxartnum = T_ARTNUM_CONST(0);
+			break;
+	}
+}
+#	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
 #endif /* NNTP_ABLE */
