@@ -3,9 +3,9 @@
  *  Module    : nntps.c
  *  Author    : E. Berkhan
  *  Created   : 2022-09-10
- *  Updated   : 2022-10-31
+ *  Updated   : 2023-09-26
  *  Notes     : simple abstraction for various TLS implementations
- *  Copyright : (c) Copyright 2022 Enrik Berkhan <Enrik.Berkhan@inka.de>
+ *  Copyright : (c) Copyright 2022-2023 Enrik Berkhan <Enrik.Berkhan@inka.de>
  *              Permission is hereby granted to copy, reproduce, redistribute
  *              or otherwise use this software  as long as: there is no
  *              monetary  profit  gained  specifically  from the use or
@@ -41,8 +41,8 @@ static SSL_CTX *openssl_ctx = NULL;
 #endif /* USE_LIBTLS */
 
 #ifdef USE_GNUTLS
-#	ifdef DEBUG
 static int verification_func(gnutls_session_t session);
+#	ifdef DEBUG
 static void log_func(int level, const char *msg);
 #	endif /* DEBUG */
 #else
@@ -77,18 +77,31 @@ tintls_init(
 
 #ifdef USE_LIBTLS
 
+	/*
+	 * libtls does not support compression, no actions needed to disable.
+	 */
+
 	libtls_config = tls_config_new();
-	if (!libtls_config)
+	if (!libtls_config) {
+		error_message(2, "tls_config_new: out of memory!\n");
 		return -ENOMEM;
+	}
 
-	if (ca_cert_file[0] == '\0')
-		ca_cert_file = tls_default_ca_cert_file();
-
-	result = tls_config_set_ca_file(libtls_config, ca_cert_file);
-	if (result != 0) {
-		tls_config_free(libtls_config);
-		libtls_config = NULL;
-		return -EINVAL;
+	/*
+	 * Only call tls_config_set_ca_file(3) if ca_cert_file has been
+	 * configured by the user. Don't use tls_default_ca_cert_file(3).
+	 * Otherwise, a behavioural difference between libretls and libressl
+	 * can be triggered.
+	 * (see https://git.causal.agency/libretls/about/#Compatibility)
+	 */
+	if (ca_cert_file[0] != '\0') {
+		result = tls_config_set_ca_file(libtls_config, ca_cert_file);
+		if (result != 0) {
+			tls_config_free(libtls_config);
+			libtls_config = NULL;
+			error_message(2, "tls_config_set_ca_file: %d!\n", result);
+			return -EINVAL;
+		}
 	}
 
 	if (insecure_nntps) {
@@ -99,6 +112,13 @@ tintls_init(
 
 #else
 #	ifdef USE_GNUTLS
+
+	/*
+	 * GnuTLS does no longer support any compression since 3.6.0, no
+	 * actions needed to disable, see:
+	 * https://www.gnutls.org/manual/gnutls.html#Compression-algorithms-and-the-record-layer
+	 */
+
 #		ifdef DEBUG
 	if (debug & DEBUG_NNTP) {
 		gnutls_global_set_log_level(2);
@@ -108,14 +128,17 @@ tintls_init(
 #		endif /* DEBUG */
 
 	result = gnutls_certificate_allocate_credentials(&tls_xcreds);
-	if (result < 0)
+	if (result < 0) {
+		error_message(2, "gnutls_certificate_allocate_credentials: out of memory!\n");
 		return -ENOMEM;
+	}
 
 	if (ca_cert_file[0] == '\0') {
 		result = gnutls_certificate_set_x509_system_trust(tls_xcreds);
 		if (result < 0) {
 			gnutls_certificate_free_credentials(tls_xcreds);
 			tls_xcreds = NULL;
+			error_message(2, "gnutls_certificate_set_x509_system_trust: %d!\n", result);
 			return -EINVAL;
 		}
 	} else {
@@ -123,6 +146,7 @@ tintls_init(
 		if (result < 0) {
 			gnutls_certificate_free_credentials(tls_xcreds);
 			tls_xcreds = NULL;
+			error_message(2, "gnutls_certificate_set_x509_trust_file: %d!\n", result);
 			return -EINVAL;
 		}
 	}
@@ -143,6 +167,14 @@ tintls_init(
 		return -ENOMEM;
 	}
 
+	/*
+	 * OpenSSL still can support compression, but this option should
+	 * already be enabled by default. We want to make sure that TLS
+	 * compression is not enabled in any case.
+	 * See e.g. RFC 8054 Section 1.1
+	 */
+	SSL_CTX_set_options(openssl_ctx, SSL_OP_NO_COMPRESSION);
+
 	if (ca_cert_file[0] == '\0') {
 		result = SSL_CTX_set_default_verify_paths(openssl_ctx);
 		if (result != 1) {
@@ -223,9 +255,8 @@ tintls_open(
 	*session_ctx = NULL;
 
 	client = tls_client();
-	if (!client) {
+	if (!client)
 		return -ENOMEM;
-	}
 
 	result = tls_configure(client, libtls_config);
 	if (result == -1) {
@@ -252,10 +283,9 @@ tintls_open(
 
 	*session_ctx = NULL;
 
-	result = gnutls_init(&client, GNUTLS_CLIENT|GNUTLS_AUTO_REAUTH|GNUTLS_POST_HANDSHAKE_AUTH);
-	if (result < 0) {
+	result = gnutls_init(&client, GNUTLS_CLIENT | GNUTLS_AUTO_REAUTH | GNUTLS_POST_HANDSHAKE_AUTH);
+	if (result < 0)
 		return -ENOMEM;
-	}
 
 	result = gnutls_server_name_set(client, GNUTLS_NAME_DNS, servername, strlen(servername));
 	if (result < 0) {
@@ -364,6 +394,7 @@ tintls_handshake(
 
 	if (result < 0) {
 		const char *err = tls_error(client);
+
 		error_message(2, "TLS handshake failed: %s!\n", err ? err : "unknown error");
 		return -EPROTO;
 	}
@@ -414,7 +445,7 @@ tintls_handshake(
 			gnutls_free(msg.data);
 		}
 
-		error_message(2, "TLS handshake failed: %s!\n", gnutls_strerror(result));
+		error_message(2, "TLS handshake failed: %s (%d)\n", gnutls_strerror(result), result);
 
 		return -EPROTO;
 	} else {
@@ -454,25 +485,25 @@ tintls_handshake(
 
 			result = gnutls_x509_crt_init(&servercert);
 			if (result < 0) {
-				error_message(1, "gnutls_x509_crt_init: %s\n", gnutls_strerror(result));
+				error_message(1, "gnutls_x509_crt_init: %s (%d)\n", gnutls_strerror(result), result);
 				goto err_cert;
 			}
 
 			result = gnutls_x509_crt_import(servercert, &raw_servercert_chain[0], GNUTLS_X509_FMT_DER);
 			if (result < 0) {
-				error_message(1, "gnutls_x509_crt_import: %s\n", gnutls_strerror(result));
+				error_message(1, "gnutls_x509_crt_import: %s (%d)\n", gnutls_strerror(result), result);
 				goto err_cert;
 			}
 
 			result = gnutls_x509_crt_get_dn3(servercert, &subject, 0);
 			if (result < 0) {
-				error_message(1, "gnutls_x509_crt_get_dn3: %s\n", gnutls_strerror(result));
+				error_message(1, "gnutls_x509_crt_get_dn3: %s (%d)\n", gnutls_strerror(result), result);
 				goto err_cert;
 			}
 
 			result = gnutls_x509_crt_get_issuer_dn3(servercert, &issuer, 0);
 			if (result < 0) {
-				error_message(1, "gnutls_x509_crt_get_issuer_dn3: %s\n", gnutls_strerror(result));
+				error_message(1, "gnutls_x509_crt_get_issuer_dn3: %s (%d)\n", gnutls_strerror(result), result);
 				goto err_cert;
 			}
 
@@ -488,7 +519,6 @@ err_cert:
 				gnutls_free(subject.data);
 			if (servercert)
 				gnutls_x509_crt_deinit(servercert);
-
 		}
 
 		desc = gnutls_session_get_desc(client);
@@ -565,7 +595,7 @@ tintls_read(
 	} while (result == TLS_WANT_POLLIN || result == TLS_WANT_POLLOUT);
 
 	return result;
-	/*NOTREACHED*/
+	/* NOTREACHED */
 #else
 #	ifdef USE_GNUTLS
 	ssize_t result = GNUTLS_E_AGAIN;
@@ -576,7 +606,7 @@ tintls_read(
 	}
 
 	return result;
-	/*NOTREACHED*/
+	/* NOTREACHED */
 #	else
 #		ifdef USE_OPENSSL
 	size_t bytes_read;
@@ -589,7 +619,7 @@ tintls_read(
 	if (result == 1)
 		return (ssize_t) bytes_read;
 
-#		endif /* USE_OPENSSL*/
+#		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
 #endif /* USE_LIBTLS */
 
@@ -615,7 +645,7 @@ tintls_write(
 	} while (result == TLS_WANT_POLLOUT || result == TLS_WANT_POLLIN);
 
 	return result;
-	/*NOTREACHED*/
+	/* NOTREACHED */
 #else
 #	ifdef USE_GNUTLS
 	ssize_t result = GNUTLS_E_AGAIN;
@@ -626,7 +656,7 @@ tintls_write(
 	}
 
 	return result;
-	/*NOTREACHED*/
+	/* NOTREACHED */
 #	else
 #		ifdef USE_OPENSSL
 	int result;
@@ -668,7 +698,7 @@ tintls_close(
 		return -EPROTO;
 
 	return result;
-	/*NOTREACHED*/
+	/* NOTREACHED */
 #else
 #	ifdef USE_GNUTLS
 	int result;
@@ -697,46 +727,47 @@ tintls_close(
 }
 
 
+/*
+ * TODO: -> lang.c
+ */
 int
 tintls_conninfo(
 	void *session_ctx,
 	FILE *fp)
 {
-#ifdef USE_LIBTLS
 	int result;
+#ifdef USE_LIBTLS
 	struct tls *client = session_ctx;
 	time_t t;
 	struct tm *tm;
 	char fmt_time[64]; /* time zone name could long... */
 
 	fprintf(fp, "\nTLS information:\n");
-	fprintf(fp,   "----------------\n");
+	fprintf(fp, "----------------\n");
 	fprintf(fp, "%s %s (strength %d)\n", tls_conn_version(client), tls_conn_cipher(client), tls_conn_cipher_strength(client));
 	fprintf(fp, "\nServer certificate information:\n");
-	fprintf(fp,   "-------------------------------\n");
+	fprintf(fp, "-------------------------------\n");
 	fprintf(fp, "Subject: %s\n", tls_peer_cert_subject(client));
-	fprintf(fp, "Issuer:  %s\n", tls_peer_cert_issuer(client));
+	fprintf(fp, "Issuer : %s\n", tls_peer_cert_issuer(client));
 
 	t = tls_peer_cert_notbefore(client);
 	tm = localtime(&t);
 	result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm); /* make format configurable? */
 	if (result < 0)
 		my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-	fprintf(fp, "Valid not before: %s\n", fmt_time);
+	fprintf(fp, txt_valid_not_before, fmt_time);
 
 	t = tls_peer_cert_notafter(client);
 	tm = localtime(&t);
 	result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm);
 	if (result < 0)
 		my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-	fprintf(fp, "Valid not after:  %s\n", fmt_time);
+	fprintf(fp, txt_valid_not_after, fmt_time);
 
-	return 0;
 #else
 
 #	ifdef USE_GNUTLS
 	int retval = -1;
-	int result;
 	gnutls_session_t client = session_ctx;
 	char *desc;
 	gnutls_datum_t msg;
@@ -749,7 +780,7 @@ tintls_conninfo(
 
 	desc = gnutls_session_get_desc(client);
 	fprintf(fp, "\nTLS information:\n");
-	fprintf(fp,   "----------------\n");
+	fprintf(fp, "----------------\n");
 	fprintf(fp, "%s\n", desc);
 	gnutls_free(desc);
 
@@ -757,12 +788,13 @@ tintls_conninfo(
 
 	if (gnutls_verification_status != 0) {
 		int type;
+
 		type = gnutls_certificate_type_get2(client, GNUTLS_CTYPE_SERVER);
 		result = gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0);
 
 		if (result == 0) {
 			fprintf(fp, "Server certificate verification FAILED:\n\t%s (%s)\n", msg.data,
-					insecure_nntps ? "tolerated as -k (insecure) requested" : "UNEXPECTED, possible BUG");
+					insecure_nntps ? "tolerated as \"-k\" (insecure) requested" : "UNEXPECTED, possible BUG");
 		} else
 			fprintf(fp, "Server certificate verification FAILED: <can't get reason>\n");
 
@@ -773,7 +805,7 @@ tintls_conninfo(
 	raw_servercert_chain = gnutls_certificate_get_peers(client, &servercert_chainlen);
 	if (servercert_chainlen > 0) {
 		fprintf(fp, "\nServer certificate information:\n");
-		fprintf(fp,   "-------------------------------\n");
+		fprintf(fp, "-------------------------------\n");
 	}
 
 	for (i = 0; i < servercert_chainlen; i++) {
@@ -786,46 +818,44 @@ tintls_conninfo(
 		fprintf(fp, "Certificate #%d\n", i);
 
 		result = gnutls_x509_crt_init(&servercert);
-		if (result < 0) {
+		if (result < 0)
 			goto err_cert;
-		}
 
 		result = gnutls_x509_crt_import(servercert, &raw_servercert_chain[i], GNUTLS_X509_FMT_DER);
-		if (result < 0) {
+		if (result < 0)
 			goto err_cert;
-		}
 
 		result = gnutls_x509_crt_get_dn3(servercert, &subject, 0);
-		if (result < 0) {
+		if (result < 0)
 			goto err_cert;
-		}
+
 		fprintf(fp, "Subject: %s\n", subject.data);
 
 		result = gnutls_x509_crt_get_issuer_dn3(servercert, &issuer, 0);
-		if (result < 0) {
+		if (result < 0)
 			goto err_cert;
-		}
-		fprintf(fp, "Issuer:  %s\n", issuer.data);
+
+		fprintf(fp, "Issuer : %s\n", issuer.data);
 
 		t = gnutls_x509_crt_get_activation_time(servercert);
-		if (t == -1) {
+		if (t == -1)
 			goto err_cert;
-		}
+
 		tm = localtime(&t);
 		result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm); /* make format configurable? */
 		if (result < 0)
 			my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-		fprintf(fp, "Valid not before: %s\n", fmt_time);
+		fprintf(fp, txt_valid_not_before, fmt_time);
 
 		t = gnutls_x509_crt_get_expiration_time(servercert);
-		if (t == -1) {
+		if (t == -1)
 			goto err_cert;
-		}
+
 		tm = localtime(&t);
 		result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm);
 		if (result < 0)
 			my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-		fprintf(fp, "Valid not after:  %s\n", fmt_time);
+		fprintf(fp, txt_valid_not_after, fmt_time);
 
 		retval = 0;
 
@@ -836,14 +866,12 @@ err_cert:
 			gnutls_free(subject.data);
 		if (servercert)
 			gnutls_x509_crt_deinit(servercert);
-
 	}
 
 	return retval;
 #	else
 
 #		ifdef USE_OPENSSL
-	int result;
 	long long_result;
 	long verification_result;
 	BIO *client = session_ctx;
@@ -855,19 +883,19 @@ err_cert:
 		return -1;
 
 	fprintf(fp, "\nTLS information:\n");
-	fprintf(fp,   "----------------\n");
+	fprintf(fp, "----------------\n");
 	fprintf(fp, "%s %s\n", SSL_get_version(ssl), SSL_get_cipher_name(ssl));
 
 	verification_result = SSL_get_verify_result(ssl);
 	if (verification_result != X509_V_OK)
 		fprintf(fp, "Server certificate verification FAILED:\n\t%s (%s)\n",
 			X509_verify_cert_error_string(verification_result),
-			insecure_nntps ? "tolerated as -k (insecure) requested" : "UNEXPECTED, possible BUG");
+			insecure_nntps ? "tolerated as \"-k\" (insecure) requested" : "UNEXPECTED, possible BUG");
 	else
 		fprintf(fp, "Server certificate verified successfully.\n");
 
 	fprintf(fp, "\nServer certificate information:\n");
-	fprintf(fp,   "-------------------------------\n");
+	fprintf(fp, "-------------------------------\n");
 
 	if (verification_result == X509_V_OK)
 		chain = SSL_get_peer_cert_chain(ssl);
@@ -877,8 +905,8 @@ err_cert:
 	if (chain) {
 		char name[128];
 		const ASN1_TIME *asn1;
-		struct tm tm;
 		int i;
+		struct tm tm;
 
 		for (i = 0; i < sk_X509_num(chain); i++) {
 			X509* cert = sk_X509_value(chain, i);
@@ -887,7 +915,7 @@ err_cert:
 				fputs("\n", fp);
 			fprintf(fp, "Certificate #%d\n", i);
 			fprintf(fp, "Subject: %s\n", X509_NAME_oneline(X509_get_subject_name(cert), name, sizeof(name)));
-			fprintf(fp, "Issuer:  %s\n", X509_NAME_oneline(X509_get_issuer_name(cert), name, sizeof(name)));
+			fprintf(fp, "Issuer : %s\n", X509_NAME_oneline(X509_get_issuer_name(cert), name, sizeof(name)));
 
 			asn1 = X509_get0_notBefore(cert);
 			result = ASN1_TIME_to_tm(asn1, &tm);
@@ -895,7 +923,7 @@ err_cert:
 				result = my_strftime(name, sizeof(name), "%Y-%m-%dT%H:%M%z", &tm); /* make format configurable? */
 				if (result < 0)
 					my_strncpy(name, "<formatting error>", sizeof(name) - 1);
-				fprintf(fp, "Valid not before: %s\n", name);
+				fprintf(fp, txt_valid_not_before, name);
 			}
 
 			asn1 = X509_get0_notAfter(cert);
@@ -904,15 +932,14 @@ err_cert:
 				result = my_strftime(name, sizeof(name), "%Y-%m-%dT%H:%M%z", &tm);
 				if (result < 0)
 					my_strncpy(name, "<formatting error>", sizeof(name) - 1);
-				fprintf(fp, "Valid not after:  %s\n", name);
+				fprintf(fp, txt_valid_not_after, name);
 			}
 		}
 	}
-
-	return 0;
 #		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
 #endif /* USE_LIBTLS */
+	return 0;
 }
 
 
@@ -930,8 +957,7 @@ show_errors(
 #endif /* USE_OPENSSL */
 
 
-#ifdef DEBUG
-#	ifdef USE_GNUTLS
+#ifdef USE_GNUTLS
 static int
 verification_func(
 	gnutls_session_t session)
@@ -950,18 +976,23 @@ verification_func(
 
 	return gnutls_verification_status;
 }
+#endif /* USE_GNUTLS */
 
+
+#ifdef DEBUG
+#	ifdef USE_GNUTLS
 static void
 log_func(
 	int level,
 	const char *msg)
 {
 	int msglen = (int) strlen(msg);
+
 	if (msglen <= 0)
 		return;
 
-	if (msg[msglen-1] == '\n')
-		msglen -= 1;
+	if (msg[msglen - 1] == '\n')
+		msglen--;
 
 	debug_print_file("NNTP", "TLS%s%.*s [%d]", logtime(), msglen, msg, level);
 }
