@@ -3,10 +3,10 @@
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
  *  Created   : 1995-09-01
- *  Updated   : 2021-03-04
+ *  Updated   : 2023-11-20
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2023 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2024 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -298,7 +298,7 @@ rfc1522_decode(
 			e = charset;
 			c++;
 			while (*c && *c != '?') {
-				/* skip over optional language tags (RFC2231, RFC5646) */
+				/* skip over optional language tags (RFC 2231, RFC 5646) */
 				if (*c == '*') {
 					while (*++c && *c != '?')
 						;
@@ -423,8 +423,7 @@ do_b_encode(
 		*(t++) = *(w++);
 	}
 
-/* if (len8 & (unsigned long) 1 && !isbetween(*w,isstruct_head)) */
-	if (len8 != len8 / 2 * 2 && !isbetween(*w, isstruct_head) && (*w))
+	if ((len8 % 2) && !isbetween(*w, isstruct_head) && (*w))
 		t--;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
@@ -454,6 +453,9 @@ which_encoding(
 	int chars = 0;
 	int schars = 0;
 	int nonprint = 0;
+#ifdef MIME_BREAK_LONG_LINES
+	char *s = w;
+#endif /* MIME_BREAK_LONG_LINES */
 
 	while (*w && isspace((unsigned char) *w))
 		w++;
@@ -473,6 +475,10 @@ which_encoding(
 			return 'B';
 		return 'Q';
 	}
+#ifdef MIME_BREAK_LONG_LINES
+	else if (strlen(s) > IMF_LINE_LEN)
+		return 'X';
+#endif /* MIME_BREAK_LONG_LINES */
 	return 0;
 }
 
@@ -574,6 +580,10 @@ rfc1522_do_encode(
 	t_bool isbroken_within = FALSE;	/* is word broken due to length restriction on encoded of word? */
 	t_bool isstruct_head = FALSE;		/* are we dealing with structured header? */
 	t_bool rightafter_ew = FALSE;
+#ifdef MIME_BREAK_LONG_LINES
+	t_bool colon_seen = FALSE;
+	t_bool long_line = FALSE;
+#endif /* MIME_BREAK_LONG_LINES */
 /*
  * the list of structured header fields where '(' and ')' are
  * treated specially in rfc 1522 encoding
@@ -593,6 +603,12 @@ rfc1522_do_encode(
 
 	t = buffer = my_malloc(bufferlen);
 	encoding = which_encoding(what);
+#ifdef MIME_BREAK_LONG_LINES
+	if (encoding == 'X') {
+		long_line = TRUE;
+		encoding = 'B';
+	}
+#endif /* MIME_BREAK_LONG_LINES */
 	ew_taken_len = strlen(charset) + 7 /* =?c?E?d?= */;
 	while (*what) {
 		if (break_long_line)
@@ -602,7 +618,11 @@ rfc1522_do_encode(
 		 * follows after the point where it's split should be encoded (i.e.
 		 * even if they are made of only 7bit chars)
 		 */
+#ifdef MIME_BREAK_LONG_LINES
+		if (contains_nonprintables(what, isstruct_head) || isbroken_within || (long_line && colon_seen)) {
+#else
 		if (contains_nonprintables(what, isstruct_head) || isbroken_within) {
+#endif /* MIME_BREAK_LONG_LINES */
 			if (encoding == 'Q') {
 				if (!quoting) {
 					snprintf(buf2, sizeof(buf2), "=?%s?%c?", charset, encoding);
@@ -767,7 +787,11 @@ rfc1522_do_encode(
 				 * if encoded word is followed by 7bit-only fragment, we need to
 				 * eliminate ' ' inserted in while-block above
 				 */
+#ifdef MIME_BREAK_LONG_LINES
+				if (!contains_nonprintables(what, isstruct_head) && !long_line) {
+#else
 				if (!contains_nonprintables(what, isstruct_head)) {
+#endif /* MIME_BREAK_LONG_LINES */
 					t--;
 					ewsize--;
 				}
@@ -781,6 +805,10 @@ rfc1522_do_encode(
 					buffer = my_realloc(buffer, bufferlen * sizeof(*buffer));
 					t = buffer + offset;
 				}
+#ifdef MIME_BREAK_LONG_LINES
+				if (*what == ':')
+					colon_seen = TRUE;
+#endif /* MIME_BREAK_LONG_LINES */
 				*t++ = *what++;		/* output word unencoded */
 			}
 			while (*what && isbetween(*what, isstruct_head)) {
@@ -817,7 +845,7 @@ every "word" were a space ... */
 		while (*c) {
 			if (isspace((unsigned char) *c)) {
 				/*
-				 * According to rfc1522, header lines containing encoded
+				 * According to RFC 1522, header lines containing encoded
 				 * words are limited to 76 chars, but if the first line is
 				 * too long (due to a long header keyword), we cannot stick
 				 * to that, since we would break the line directly after the
@@ -866,6 +894,8 @@ rfc1522_encode(
 	 */
 #ifdef MIME_BREAK_LONG_LINES
 	t_bool break_long_line = TRUE;
+	/* silence compiler warning (unused parameter) */
+	(void) ismail;
 #else
 	/*
 	 * Even if MIME_BREAK_LONG_LINES is NOT defined, long headers in mail
@@ -924,9 +954,8 @@ do_rfc15211522_encode(
 		/*
 		 * TODO: - what about 8bit chars in the mentioned headers
 		 *         when !allow_8bit_header?
-		 *       - what about lines longer 998 octets?
 		 */
-		if (allow_8bit_header || (!strncasecmp(header, "References: ", 12) || !strncasecmp(header, "Message-ID: ", 12) || !strncasecmp(header, "Date: ", 6) || !strncasecmp(header, "Newsgroups: ", 12) || !strncasecmp(header, "Distribution: ", 14) || !strncasecmp(header, "Followup-To: ", 13) || !strncasecmp(header, "X-Face: ", 8) || !strncasecmp(header, "Cancel-Lock: ", 13) || !strncasecmp(header, "Cancel-Key: ", 12)))
+		if (allow_8bit_header || (!strncasecmp(header, "References: ", 12) || !strncasecmp(header, "Message-ID: ", 12) || !strncasecmp(header, "Date: ", 6) || !strncasecmp(header, "Newsgroups: ", 12) || !strncasecmp(header, "Distribution: ", 14) || !strncasecmp(header, "Followup-To: ", 13) || !strncasecmp(header, "X-Face: ", 8) || !strncasecmp(header, "Cancel-Lock: ", 13) || !strncasecmp(header, "Cancel-Key: ", 12) || !strncasecmp(header, "Path: ", 6)))
 			fputs(header, g);
 		else {
 			char *p;
@@ -1000,13 +1029,13 @@ do_rfc15211522_encode(
 		 */
 		if (mime_headers_needed) {
 			if (contains_headers)
-				fprintf(f, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
+				fprintf(f, txt_mime_version, MIME_SUPPORTED_VERSION);
 #ifdef CHARSET_CONVERSION
-			fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[mmnwcharset]);
+			fprintf(f, txt_mime_hdr_c_type_text_plain_charset, txt_mime_charsets[mmnwcharset]);
 #else
-			fprintf(f, "Content-Type: text/plain; charset=%s\n", tinrc.mm_charset);
+			fprintf(f, txt_mime_hdr_c_type_text_plain_charset, tinrc.mm_charset);
 #endif /* CHARSET_CONVERSION */
-			fprintf(f, "Content-Transfer-Encoding: %s\n", mime_encoding);
+			fprintf(f, txt_mime_hdr_c_transfer_encoding, mime_encoding);
 		}
 	}
 	fputc('\n', f);
@@ -1248,7 +1277,7 @@ compose_mail_mime_forwarded(
 		if (*line != '\0')
 			fprintf(fp, "%s\n", line);
 	}
-	fprintf(fp, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
+	fprintf(fp, txt_mime_version, MIME_SUPPORTED_VERSION);
 	rewind(entityfp);
 	copy_fp(entityfp, fp);
 
@@ -1277,9 +1306,9 @@ compose_message_rfc822(
 	*is_8bit = contains_8bit_characters(articlefp);
 
 	/* Header: CT, CD, CTE */
-	fprintf(fp, "Content-Type: message/rfc822\n");
-	fprintf(fp, "Content-Disposition: inline\n");
-	fprintf(fp, "Content-Transfer-Encoding: %s\n", *is_8bit ? txt_8bit : txt_7bit);
+	fprintf(fp, "%s", txt_mime_hdr_c_type_msg_rfc822);
+	fprintf(fp, "%s", txt_mime_hdr_c_disposition_inline);
+	fprintf(fp, txt_mime_hdr_c_transfer_encoding, *is_8bit ? txt_8bit : txt_7bit);
 	fputc('\n', fp);
 
 	/* Body: articlefp */
@@ -1301,7 +1330,7 @@ compose_multipart_mixed(
 {
 	FILE *fp;
 	FILE *messagefp;
-	char boundary[MIME_BOUNDARY_SIZE];
+	char *boundary;
 	t_bool requires_8bit;
 
 	if ((fp = tmpfile()) == NULL)
@@ -1314,42 +1343,41 @@ compose_multipart_mixed(
 	}
 
 	requires_8bit = (requires_8bit || contains_8bit_characters(textfp));
+	boundary = my_malloc(MIME_BOUNDARY_SIZE);
 
 	/*
 	 * Header: CT with multipart boundary, CTE
-	 * TODO: -> lang.c
 	 */
 	generate_mime_boundary(boundary, textfp, articlefp);
-	fprintf(fp, "Content-Type: multipart/mixed; boundary=\"%s\"\n", boundary);
-	fprintf(fp, "Content-Transfer-Encoding: %s\n\n", requires_8bit ? txt_8bit : txt_7bit);
+	fprintf(fp, txt_mime_hdr_c_type_multipart_mixed, boundary);
+	fprintf(fp, txt_mime_hdr_c_transfer_encoding, requires_8bit ? txt_8bit : txt_7bit);
+	fputc('\n', fp);
 
 	/*
 	 * preamble
-	 * TODO: -> lang.c
 	 */
-	fprintf(fp, _("This message has been composed in the 'multipart/mixed' MIME-format. If you\n\
-are reading this prefix, your mail reader probably has not yet been modified\n\
-to understand the new format, and some of what follows may look strange.\n\n"));
+	fprintf(fp, "%s", _(txt_mime_preamble_multipart_mixed));
 
 	/*
 	 * Body: boundary+text, message/rfc822 part, closing boundary
 	 */
 	/* text */
-	fprintf(fp, "--%s\n", boundary);
+	fprintf(fp, txt_mime_boundary, boundary);
 	rewind(textfp);
 	copy_fp(textfp, fp);
 	fputc('\n', fp);
 
 	/* message/rfc822 part */
-	fprintf(fp, "--%s\n", boundary);
+	fprintf(fp, txt_mime_boundary, boundary);
 	rewind(messagefp);
 	copy_fp(messagefp, fp);
 	fclose(messagefp);
 	fputc('\n', fp);
 
 	/* closing boundary */
-	fprintf(fp, "--%s--\n", boundary);
+	fprintf(fp, txt_mime_boundary_end, boundary);
 	/* TODO: insert an epilogue here? */
+	free(boundary);
 	return fp;
 }
 
