@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2022-08-29
+ *  Updated   : 2023-10-29
  *  Notes     :
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -132,7 +132,8 @@ static t_partl *part_list;
 int
 check_start_save_any_news(
 	int function,
-	t_bool catchup)
+	t_bool catchup,
+	int num_cmd_line_groups)
 {
 	FILE *artfp, *savefp;
 	FILE *fp_log = (FILE *) 0;
@@ -142,13 +143,11 @@ check_start_save_any_news(
 	char logfile[PATH_LEN], savefile[PATH_LEN];
 	char subject[HEADER_LEN];
 	int group_count = 0;
-	int i, j;
+	int i, j, k;
 	int art_count, hot_count;
 	int saved_arts = 0;					/* Total # saved arts */
 	struct t_article *art;
 	struct t_group *group;
-	t_bool log_opened = TRUE;
-	t_bool print_first = (t_bool) verbose;
 	t_bool unread_news = FALSE;
 	time_t epoch;
 
@@ -173,7 +172,6 @@ check_start_save_any_news(
 				perror_message(_(txt_cannot_open), logfile);
 				fp_log = stdout;
 				verbose = FALSE;
-				log_opened = FALSE;
 			}
 			fprintf(fp_log, "To: %s\n", userid);
 			(void) time(&epoch);
@@ -185,6 +183,7 @@ check_start_save_any_news(
 			break;
 	}
 
+	k = num_cmd_line_groups;
 	/*
 	 * For each group we subscribe to...
 	 */
@@ -199,12 +198,26 @@ check_start_save_any_news(
 		 */
 		selmenu.curr = i;
 
-		if (group->bogus || !group->subscribed)
+		/*
+		 * read_during_session is missued as indicator for a cmdline group
+		 *
+		 * TODO: log "skipped" groups with verbose > 1?
+		 */
+		if (group->bogus || (!k && !group->subscribed) || (k && !group->read_during_session))
 			continue;
 
+		if (k && group->read_during_session) {
+			k--; /* just to make the test above faster once we've done the cmd-line grpups */
+			group->read_during_session = FALSE; /* reset in case we do NOT enter the group later */
+		}
+
 		if (function == MAIL_ANY_NEWS || function == SAVE_ANY_NEWS) {
-			if (!group->attribute->batch_save)
+			if (!group->attribute->batch_save) { /* TODO: string -> lang.c */
+				if (verbose > 1 && function == MAIL_ANY_NEWS)
+					fprintf(fp_log, "Skipped %s", group->name);
+
 				continue;
+			}
 
 			group_count++;
 			snprintf(buf, sizeof(buf), _(txt_saved_groupname), group->name);
@@ -244,10 +257,6 @@ check_start_save_any_news(
 
 			switch (function) {
 				case CHECK_ANY_NEWS:
-					if (print_first) {
-						my_fputc('\n', stdout);
-						print_first = FALSE;
-					}
 					if (!verbose && !catchup) /* we don't need details */
 						return NEWS_AVAIL_EXIT;
 					art_count++;
@@ -362,7 +371,7 @@ check_start_save_any_news(
 			if (verbose)
 				wait_message(0, "%s", buf);
 
-			if (log_opened) {
+			if (fp_log != stdout) {
 				fclose(fp_log);
 				if (verbose)
 					wait_message(0, _(txt_mail_log_to), (function == MAIL_ANY_NEWS ? mail_news_user : userid));
@@ -390,54 +399,55 @@ open_save_filename(
 {
 	FILE *fp;
 	char keyappend[MAXKEYLEN], keyoverwrite[MAXKEYLEN], keyquit[MAXKEYLEN];
-	char mode[3];
 	struct stat st;
 	t_function func;
 
-	strcpy(mode, "a+");
+	if ((fp = fopen(path, "a+")) == NULL) {
+		perror_message(_(txt_cannot_open_for_saving), path);
+		return NULL;
+	}
 
-	/*
-	 * Mailboxes will always be appended to
-	 */
-	if (!mbox && stat(path, &st) != -1) {
+	if (fstat(fileno(fp), &st) != -1) {
 		/*
 		 * Admittedly a special case hack, but it saves failing later on
 		 */
 		if (S_ISDIR(st.st_mode)) {
 			wait_message(2, _(txt_cannot_write_to_directory), path);
+			fclose(fp);
 			return NULL;
 		}
 /* TODO: will this get called every art? Should only be done once/batch */
-/* TODO: or add an option for defaulting on all future queries */
+/* TODO: or add an option for defaulting on all future queries (e.g. A/O) */
 /* TODO: 'truncate' path if query exceeds screen-width */
-		func = prompt_slk_response((tinrc.default_save_mode == 'a' ? SAVE_APPEND_FILE : SAVE_OVERWRITE_FILE),
-				save_append_overwrite_keys,
-				_(txt_append_overwrite_quit), path,
-				PrintFuncKey(keyappend, SAVE_APPEND_FILE, save_append_overwrite_keys),
-				PrintFuncKey(keyoverwrite, SAVE_OVERWRITE_FILE, save_append_overwrite_keys),
-				PrintFuncKey(keyquit, GLOBAL_QUIT, save_append_overwrite_keys));
+		if (!mbox && (S_ISREG(st.st_mode) && st.st_size > 0L)) { /* Mailboxes will always be appended to; empty files will be "overwritten" */
+			func = prompt_slk_response((tinrc.default_save_mode == 'a' ? SAVE_APPEND_FILE : SAVE_OVERWRITE_FILE),
+					save_append_overwrite_keys,
+					_(txt_append_overwrite_quit), path,
+					PrintFuncKey(keyappend, SAVE_APPEND_FILE, save_append_overwrite_keys),
+					PrintFuncKey(keyoverwrite, SAVE_OVERWRITE_FILE, save_append_overwrite_keys),
+					PrintFuncKey(keyquit, GLOBAL_QUIT, save_append_overwrite_keys));
 
-		switch (func) {
-			case SAVE_OVERWRITE_FILE:
-				strcpy(mode, "w");
-				break;
+			switch (func) {
+				case SAVE_OVERWRITE_FILE:
+					tinrc.default_save_mode = 'o';
+					if (!ftruncate(fileno(fp), 0L))
+						(void) fseek(fp, 0L, SEEK_SET);
+					break;
 
-			case GLOBAL_ABORT:
-			case GLOBAL_QUIT:
-				wait_message(1, _(txt_art_not_saved));
-				return NULL;
+				case GLOBAL_ABORT:
+				case GLOBAL_QUIT:
+					fclose(fp);
+					wait_message(1, _(txt_art_not_saved));
+					return NULL;
 
-			default:	/* SAVE_APPEND_FILE */
-				break;
+				default:	/* SAVE_APPEND_FILE */
+					tinrc.default_save_mode = 'a';
+					break;
+			}
 		}
-		if (func == SAVE_OVERWRITE_FILE)
-			tinrc.default_save_mode = 'o';
-		else
-			tinrc.default_save_mode = 'a';
-	}
-
-	if ((fp = fopen(path, mode)) == NULL) {
-		error_message(2, _(txt_cannot_open_for_saving), path);
+	} else { /* fstat() failed */
+		perror_message(_(txt_cannot_open_for_saving), path);
+		fclose(fp);
 		return NULL;
 	}
 
@@ -492,11 +502,8 @@ save_and_process_art(
 	 * saves. Multiple file saves append a .NNN sequence number to the path
 	 * This is backward-contemptibility with older versions of tin
 	 */
-	if (!is_mailbox && max > 1) {
-		const char suffixsep = '.';
-
-		sprintf(&path[strlen(path)], "%c%03d", suffixsep, num_save + 1);
-	}
+	if (!is_mailbox && max > 1)
+		sprintf(&path[strlen(path)], ".%03d", num_save + 1);
 
 /* fprintf(stderr, "save_and_process_art expanded path now=(%s)\n", path); */
 
@@ -541,7 +548,7 @@ save_and_process_art(
 	if (num_save == max_save - 1)
 		expand_save();
 	save[num_save].path = my_strdup(path);
-	save[num_save].file = strrchr(save[num_save].path, DIRSEP) + 1;	/* ptr to filename portion */
+	save[num_save].file = strrchr(save[num_save].path, '/') + 1;	/* ptr to filename portion */
 	save[num_save].mailbox = CAST_BOOL(is_mailbox);
 /* fprintf(stderr, "SAPA (%s) (%s) mbox=%s\n", save[num_save].path, save[num_save].file, bool_unparse(save[num_save].mailbox)); */
 	num_save++;			/* NB: num_save is bumped here only */
@@ -570,41 +577,38 @@ save_and_process_art(
  * Create the supplied path. Create intermediate directories as needed
  * Don't create the last component (which would be the filename) unless the
  * path is / terminated.
- * Return FALSE if it somehow fails.
+ * Return errno if it somehow fails.
  */
-t_bool
+int
 create_path(
 	const char *path)
 {
 	char *buf, *p;
+	int pe = 0;
 	struct stat st;
 
-	if (!strlen(path))
-		return FALSE;
+	if (!*path || !*(path + 1))
+		return ENOTDIR;
 
-	buf = my_strdup(path);
-	p = buf + 1;
+	p = buf = my_strdup(path);
+	p++;
 
-	if (!strlen(p)) {
-		free(buf);
-		return FALSE;
-	}
-
-	while ((p = strchr(p, DIRSEP)) != NULL) {
+	while ((p = strchr(p, '/')) != NULL) {
 		*p = '\0';
 		if (stat(buf, &st) == -1) {
 			if (my_mkdir(buf, (mode_t) (S_IRWXU|S_IRUGO|S_IXUGO)) == -1) {
 				if (errno != EEXIST) {
+					pe = errno;
 					perror_message(_(txt_cannot_create), buf);
 					free(buf);
-					return FALSE;
+					return pe;
 				}
 			}
 		}
-		*p++ = DIRSEP;
+		*p++ = '/';
 	}
 	free(buf);
-	return TRUE;
+	return pe;
 }
 
 
@@ -661,8 +665,8 @@ generate_savepath(
 		return NULL;
 	}
 
-	if (!(create_path(savepath))) {
-		error_message(2, _(txt_cannot_open_for_saving), savepath);
+	if ((errno = create_path(savepath)) != 0) {
+		perror_message(_(txt_cannot_open_for_saving), savepath);
 		free(savepath);
 		return NULL;
 	}
@@ -831,7 +835,7 @@ post_process_uud(
 			/* item->mimetype seems not to be available for uudecoded files etc */
 			if (curr_group->attribute->post_process_view) {
 				joinpath(path, sizeof(path), file_out_dir, item->filename);
-				view_file(path, strrchr(path, DIRSEP) + 1);
+				view_file(path, strrchr(path, '/') + 1);
 			}
 		} else {
 			errors++;
@@ -920,7 +924,7 @@ post_process_uud(
 
 						filename = name;
 						expand_save_filename(path, sizeof(path), filename);
-						filename = strrchr(path, DIRSEP) + 1;	/* ptr to filename portion */
+						filename = strrchr(path, '/') + 1;	/* ptr to filename portion */
 						if ((fp_out = fopen(path, "w")) == NULL) {
 							perror_message(_(txt_cannot_open), path);
 							fclose(fp_in);
@@ -1010,7 +1014,6 @@ post_process_uud(
 		my_printf(_(txt_uu_error_decode), filename, _(txt_uu_error_no_end));
 		my_printf(cCRLF);
 	}
-	return;
 }
 
 
@@ -1515,6 +1518,7 @@ static void
 show_attachment_page(
 	void)
 {
+	char *attach_line;
 	char buf[BUFSIZ];
 	const char *charset;
 	int i, tmp_len, max_depth;
@@ -1530,11 +1534,11 @@ show_attachment_page(
 	info_len = max_depth = 0;
 	for (i = 0; i < attmenu.max; ++i) {
 		part = get_part(i);
-		snprintf(buf, sizeof(buf), _(txt_attachment_lines), part->line_count);
-		tmp_len = strwidth(buf);
 		charset = get_param(part->params, "charset");
-		snprintf(buf, sizeof(buf), "  %s/%s, %s, %s%s", content_types[part->type], part->subtype, content_encodings[part->encoding], charset ? charset : "", charset ? ", " : "");
-		tmp_len += strwidth(buf);
+		attach_line = build_attach_line(part, 0, cCOLS - 2, 0, NULL, charset);
+		snprintf(buf, sizeof(buf), "  %s", attach_line);
+		FreeIfNeeded(attach_line);
+		tmp_len = strwidth(buf);
 		if (tmp_len > info_len)
 			info_len = tmp_len;
 
@@ -1790,6 +1794,7 @@ static void
 build_attachment_line(
 	int i)
 {
+	char *attach_line;
 	char *sptr;
 	const char *name;
 	const char *charset;
@@ -1827,12 +1832,6 @@ build_attachment_line(
 	}
 
 	charset = get_param(part->params, "charset");
-	snprintf(buf2, sizeof(buf2), _(txt_attachment_lines), part->line_count);
-	/* TODO: make the layout configurable? */
-	if (!strcmp(content_types[part->type], "text"))
-		snprintf(buf, sizeof(buf), "  %s/%s, %s, %s%s%s", content_types[part->type], part->subtype, content_encodings[part->encoding], charset ? charset : "", charset ? ", " : "", buf2);
-	else
-		snprintf(buf, sizeof(buf), "  %s/%s, %s, %s", content_types[part->type], part->subtype, content_encodings[part->encoding], buf2);
 	if (part->depth > 0) {
 		treelen = cCOLS - 13 - info_len - namelen;
 		tree = build_tree(part->depth, treelen, i);
@@ -1843,6 +1842,10 @@ build_attachment_line(
 	if (namelen + len + info_len + 8 <= cCOLS)
 		namelen = cCOLS - 8 - info_len - len;
 
+	attach_line = build_attach_line(part, 0, info_len - 2, 0, NULL, charset);
+	snprintf(buf, sizeof(buf), "  %s", attach_line);
+	FreeIfNeeded(attach_line);
+
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	tmpname = spart(name, namelen, TRUE);
 	tmpbuf = spart(buf, info_len, TRUE);
@@ -2028,9 +2031,8 @@ tag_pattern(
 			lptr->tagged = ++num_of_tagged_parts;
 	}
 
-	if (tinrc.wildcard) {
+	if (tinrc.wildcard)
 		regex_cache_destroy(&cache);
-	}
 }
 
 
@@ -2298,7 +2300,7 @@ process_part(
 						if (what != SAVE && what != SAVE_TAGGED && !strncmp(content_types[part->type], "text", 4)) {
 							line_len = (size_t) count;
 							conv_buf = my_strdup(buf2);
-							network_charset = get_param(part->params, "charset");
+							network_charset = validate_charset(get_param(part->params, "charset"));
 							process_charsets(&conv_buf, &line_len, network_charset ? network_charset : "US-ASCII", tinrc.mm_local_charset, FALSE);
 							strncpy(buf2, conv_buf, sizeof(buf2) - 1);
 							count = (int) strlen(buf2);
@@ -2323,7 +2325,7 @@ process_part(
 						if (what != SAVE && what != SAVE_TAGGED && !strncmp(content_types[part->type], "text", 4)) {
 							conv_buf = my_strdup(buf);
 							line_len = strlen(conv_buf);
-							network_charset = get_param(part->params, "charset");
+							network_charset = validate_charset(get_param(part->params, "charset"));
 							process_charsets(&conv_buf, &line_len, network_charset ? network_charset : "US-ASCII", tinrc.mm_local_charset, FALSE);
 							strncpy(buf, conv_buf, sizeof(buf) - 1);
 							free(conv_buf);
