@@ -3,10 +3,10 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2022-10-13
+ *  Updated   : 2023-11-15
  *  Notes     :
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -302,6 +302,11 @@ selection_page(
 				show_selection_page();
 				break;
 
+			case GLOBAL_CONNECTION_INFO:
+				show_connection_page();
+				show_selection_page();
+				break;
+
 #ifdef NNTP_ABLE
 			case GLOBAL_LOOKUP_MESSAGEID:
 				switch (show_article_by_msgid(NULL)) {
@@ -414,7 +419,7 @@ selection_page(
 				 * as we effectively do a yank out on each change, set yanked_out accordingly
 				 */
 				yanked_out = TRUE;
-				wait_message(0, _(txt_reading_groups), (tinrc.show_only_unread_groups) ? _("unread") : _("all"));
+				wait_message(0, _(txt_reading_groups), (tinrc.show_only_unread_groups) ? _(txt_unread) : _(txt_all));
 
 				toggle_my_groups(NULL);
 				show_selection_page();
@@ -514,7 +519,7 @@ selection_page(
 #if 1 /* TODO: fix the rest of the code so we don't need this anymore */
 						/*
 						 * this is a gross hack to avoid a crash in the
-						 * CHARSET_CONVERSION conversion case in new_part()
+						 * CHARSET_CONVERSION case in new_part()
 						 * which currently relies on CURR_GROUP
 						 */
 						selmenu.curr = my_group_add(buf, FALSE);
@@ -611,15 +616,15 @@ show_selection_page(
 
 	if (use_nntps) {
 		if (insecure_nntps)
-			secflag=_("[k]");
+			secflag=_(txt_selection_flag_insecure);
 		else
-			secflag=_("[T]");
+			secflag=_(txt_selection_flag_secure);
 	}
 
 	if (read_news_via_nntp)
-		snprintf(buf, sizeof(buf), "%s (%s%s  %d%s)", _(txt_group_selection), nntp_server, secflag, selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
+		snprintf(buf, sizeof(buf), "%s (%s%s  %d%s)", _(txt_group_selection), nntp_server, secflag, selmenu.max, (tinrc.show_only_unread_groups ? _(txt_selection_flag_only_unread) : ""));
 	else
-		snprintf(buf, sizeof(buf), "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
+		snprintf(buf, sizeof(buf), "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(txt_selection_flag_only_unread) : ""));
 
 	if (selmenu.curr < 0)
 		selmenu.curr = 0;
@@ -1405,6 +1410,8 @@ subscribe_pattern(
 {
 	char buf[LEN];
 	int i, subscribe_num = 0;
+	size_t groups_size = 100;
+	struct t_group **groups;
 
 	if (!num_active || no_write)
 		return;
@@ -1414,14 +1421,20 @@ subscribe_pattern(
 		return;
 	}
 
+	groups = my_malloc(groups_size * sizeof(struct t_group *));
+
 	wait_message(0, "%s", message);
 
 	for_each_group(i) {
 		if (match_group_list(active[i].name, buf)) {
 			if (active[i].subscribed != (state != FALSE)) {
 				spin_cursor();
+				if ((size_t) subscribe_num == groups_size) {
+					groups_size <<= 1;
+					groups = my_realloc(groups, groups_size * sizeof(struct t_group *));
+				}
+				groups[subscribe_num] = &active[i];
 				/* If found and group is not subscribed add it to end of my_group[]. */
-				subscribe(&active[i], SUB_CHAR(state), TRUE);
 				if (state) {
 					my_group_add(active[i].name, FALSE);
 					grp_mark_unread(&active[i]);
@@ -1431,6 +1444,10 @@ subscribe_pattern(
 		}
 	}
 
+	bulk_subscribe(groups, subscribe_num, SUB_CHAR(state), TRUE);
+
+	free(groups);
+
 	if (subscribe_num) {
 		toggle_my_groups(NULL);
 		show_selection_page();
@@ -1471,7 +1488,7 @@ static void
 select_read_group(
 	void)
 {
-	struct t_group *currgrp;
+	static struct t_group *currgrp;
 
 	if (!selmenu.max || selmenu.curr == -1) {
 		info_message(_(txt_no_groups));
@@ -1648,9 +1665,8 @@ show_article_by_msgid(
 	t_bool tmp_cache_overview_files;
 	t_bool tmp_show_only_unread_arts;
 
-	if (!(read_news_via_nntp && !read_saved_news)) {
+	if (!(read_news_via_nntp && !read_saved_news))
 		return LOOKUP_UNAVAIL;
-	}
 
 	if (messageid) {
 		idptr = messageid;
