@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2021-02-23
+ *  Updated   : 2023-08-03
  *  Notes     :
  *
- * Copyright (c) 1997-2022 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2023 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -68,13 +68,13 @@
 	static wchar_t *my_wcsdup(const wchar_t *wstr);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
+
 /*
  * special ltoa()
  * converts value into a string with a maxlen of digits (usually should be
  * >=4), last char may be one of the following:
  * 'k'ilo, 'M'ega, 'G'iga, 'T'era, 'P'eta, 'E'xa, 'Z'etta, 'Y'otta,
- * 'X'ona, 'W'eka, 'V'unda, 'U'da (these last 4 are no official SI-prefixes)
- * or 'e' if an error occurs
+ * 'R'onna, 'Q'uetta or 'e' if an error occurs
  */
 char *
 tin_ltoa(
@@ -82,7 +82,7 @@ tin_ltoa(
 	int digits)
 {
 	static char buffer[64];
-	static const char power[] = { ' ', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'X', 'W', 'V', 'U', '\0' };
+	static const char power[] = { ' ', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q', '\0' };
 	int len;
 	size_t i = 0;
 
@@ -259,7 +259,7 @@ mystrcat(
  */
 int
 my_tolower(
-int c)
+	int c)
 {
 #ifdef TOLOWER_BROKEN
 	if (c >= 'A' && c <= 'Z')
@@ -274,7 +274,7 @@ int c)
 
 int
 my_toupper(
-int c)
+	int c)
 {
 #ifdef TOUPPER_BROKEN
 	if (c >= 'a' && c <= 'z')
@@ -527,7 +527,7 @@ strsep(
 
 
 /*
- * str_trim - leading and trailing whitespace
+ * str_trim - in-place string trim leading and trailing whitespace
  *
  * INPUT:  string  - string to trim
  *
@@ -539,29 +539,32 @@ char *
 str_trim(
 	char *string)
 {
-	char *rp;		/* reading string pointer */
-	char *wp;		/* writing string pointer */
-	char *ls;		/* last space */
+	char *rp, *wp, *ep;
+	size_t s;
 
 	if (string == NULL)
 		return NULL;
 
-	for (rp = wp = ls = string; isspace((int) *rp); rp++)		/* Skip leading space */
+	if (!(s = strlen(string)))
+		return string;
+
+	/* remove training spaces */
+	ep = string + s - 1;
+    while (ep >= string && isspace((int) *ep))
+        ep--;
+    *(ep + 1) = '\0';
+
+	/* skip leading space */
+	for (rp = wp = string; isspace((int) *rp); rp++)
 		;
 
-	while (*rp) {
-		if (isspace((int) *rp)) {
-			if (ls == NULL)		/* Remember last written space */
-				ls = wp;
-		} else
-			ls = NULL;			/* It wasn't the last space */
-		*wp++ = *rp++;
-	}
+	/* copy if required to keep address */
+	if (rp != string) {
+		while (*rp)
+			*wp++ = *rp++;
 
-	if (ls)						/* ie, there is trailing space */
-		*ls = '\0';
-	else
 		*wp = '\0';
+	}
 
 	return string;
 }
@@ -1040,7 +1043,7 @@ abbr_groupname(
 
 	return new_grpname;
 }
-#endif /* MULTIBYTE_ABLE && !NOLOCALE */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 
 /*
@@ -1548,7 +1551,7 @@ parse_format_string(
 				*d_fmt = '\0';
 				in++;
 			} else {
-				out -= 1;
+				out--;
 				*out++ = *in;
 				continue;
 			}
@@ -1577,7 +1580,7 @@ parse_format_string(
 				/* Date */
 				if (cCOLS > min_cols && (!(flags & DATE) && (signal_context == cGroup || signal_context == cThread))) {
 					flags |= DATE;
-					if (strlen(tmp_date_str))
+					if (*tmp_date_str)
 						strcpy(fmt->date_str, tmp_date_str);
 					else
 						STRCPY(fmt->date_str, curr_group->attribute->date_format);
@@ -1586,9 +1589,9 @@ parse_format_string(
 					if (my_strftime(buf, LEN - 1, fmt->date_str, localtime(&tmptime))) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 						if ((wtmp = char2wchar_t(buf)) != NULL) {
-							if (wcstombs(tmp, wtmp, sizeof(tmp) - 1) != (size_t) -1) {
+							if (wcstombs(tmp, wtmp, sizeof(tmp) - 1) != (size_t) -1)
 								fmt->len_date = (size_t) strwidth(tmp);
-							}
+
 							free(wtmp);
 						}
 #else
@@ -1621,9 +1624,9 @@ parse_format_string(
 				/* From */
 				if (!(flags & FROM) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= FROM;
-					if (len) {
+					if (len)
 						fmt->len_from = len;
-					}
+
 				} else
 					out -= 2;
 				break;
@@ -1632,9 +1635,9 @@ parse_format_string(
 				/* Newsgroup name */
 				if (cCOLS > min_cols && !(flags & GRP_NAME) && signal_context == cSelect) {
 					flags |= GRP_NAME;
-					if (len) {
+					if (len)
 						fmt->len_grpname = len;
-					}
+
 					fmt->len_grpname_dsc = (len2 ? len2 : 32);
 				} else
 					out -= 2;
