@@ -3,10 +3,10 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2022-10-27
+ *  Updated   : 2023-10-17
  *  Notes     :
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -247,7 +247,10 @@ build_tline(
 				if (!thrd_fmt.mark_offset)
 					thrd_fmt.mark_offset = (size_t) (mark_offset = strwidth(buffer) + 2);
 				if (art->tagged) {
-					strcat(buffer, " ");
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+					if (art_mark_width > 1)
+						strcat(buffer, " ");
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 					strcat(buffer, tin_ltoa(art->tagged, 3));
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 					mark[0] = L'\0';
@@ -643,6 +646,29 @@ thread_page(
 				ret_code = enter_pager(find_response(thread_basenote, thdmenu.curr), FALSE, THREAD_LEVEL);
 				break;
 
+/*
+			case THREAD_FOLLOWUP_QUOTE_HEADERS:
+			may need
+					if (func == THREAD_FOLLOWUP_QUOTE_HEADERS)
+						resize_article(TRUE, &pgart);
+			but as '^W' is already taken by MARK_FEED_UNREAD
+			we leave that function out for now
+*/
+			case THREAD_FOLLOWUP_QUOTE:
+			case THREAD_FOLLOWUP:
+				if (can_post || group->attribute->mailing_list != NULL) {
+					int ret;
+
+					n = find_response(thread_basenote, thdmenu.curr);
+					ret = art_open(TRUE, &arts[n], group, &pgart, TRUE, _(txt_reading_article));
+					if (ret != ART_UNAVAILABLE && ret != ART_ABORT && n >= 0) {
+						post_response(group->name, n, (func == THREAD_FOLLOWUP_QUOTE) ? TRUE : FALSE, FALSE, FALSE);
+						show_thread_page();
+					}
+					art_close(&pgart);
+				}
+				break;
+
 			case THREAD_READ_NEXT_ARTICLE_OR_THREAD:
 				ret_code = thread_tab_pressed();
 				break;
@@ -1514,7 +1540,7 @@ make_prefix(
 	buf[prefix_ptr] = TREE_HORIZ;
 	buf[--prefix_ptr] = (has_sibling(art) ? TREE_VERT_RIGHT : TREE_UP_RIGHT);
 
-	for (ptr = art->parent; prefix_ptr > 1; ptr = ptr->parent) {
+	for (ptr = art->parent; ptr && prefix_ptr > 1; ptr = ptr->parent) {
 		if (IS_EXPIRED(ptr))
 			continue;
 		buf[--prefix_ptr] = TREE_BLANK;
